@@ -27,9 +27,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 27984 $
+ * $Revision: 28445 $
  * $Author: rjongbloed $
- * $Date: 2012-07-10 03:24:24 -0500 (Tue, 10 Jul 2012) $
+ * $Date: 2012-10-02 20:11:02 -0500 (Tue, 02 Oct 2012) $
  */
 
 #ifndef OPAL_OPAL_CONNECTION_H
@@ -998,18 +998,27 @@ class OpalConnection : public PSafeObject
     );
 
 #if OPAL_FAX
-    /**Switch to/from FAX mode.
+    /**Switch to/from T.38 mode.
       */
     virtual bool SwitchFaxMediaStreams(
-      bool enableFax  ///< Enable FAX or return to audio mode
+      bool toT38  ///< Enable T.38 or return to audio mode
     );
 
-    /**Indicate status of switch to/from FAX mode.
+    /**Indicate status of switch to/from T.38 mode.
 
        Default behaviour does nothing.
       */
     virtual void OnSwitchedFaxMediaStreams(
-      bool enabledFax  ///< Enabled FAX or audio mode
+      bool toT38,       ///< Were switching to T.38 or audio mode
+      bool success      ///< Switch succeeded or failed
+    );
+
+    /**Indicate remote is requesting switch to/from T.38 fax mode.
+
+       Default behaviour calls the same function on the other connection.
+      */
+    virtual bool OnSwitchingFaxMediaStreams(
+      bool toT38   ///< Is switching to T.38 or audio mode
     );
 #endif
 
@@ -1224,11 +1233,34 @@ class OpalConnection : public PSafeObject
     ) const;
 #endif
 
-    /**Set the volume (gain) for the audio media channel to the specified percentage.
+    /**Set the volume (gain) for the audio media channel.
+       The volume range is 0 == muted, 100 == LOUDEST.
       */
     virtual PBoolean SetAudioVolume(
-      PBoolean source,                  ///< true for source (microphone), false for sink (speaker)
-      unsigned percentage           ///< Gain, 0=silent, 100=maximun
+      PBoolean source,        ///< true for source (microphone), false for sink (speaker)
+      unsigned percentage     ///< Gain, 0=silent, 100=maximun
+    );
+
+    /**Get  the volume (gain) for the audio media channel.
+       The volume range is 0 == muted, 100 == LOUDEST.
+      */
+    virtual PBoolean GetAudioVolume(
+      PBoolean source,        ///< true for source (microphone), false for sink (speaker)
+      unsigned & percentage   ///< Gain, 0=silent, 100=maximun
+    );
+
+    /**Set the mute state for the audio media channel.
+      */
+    virtual bool SetAudioMute(
+      bool source,        ///< true for source (microphone), false for sink (speaker)
+      bool mute           ///< Flag for muted audio
+    );
+
+    /**Get the mute state for the audio media channel.
+      */
+    virtual bool GetAudioMute(
+      bool source,        ///< true for source (microphone), false for sink (speaker)
+      bool & mute         ///< Flag for muted audio
     );
 
     /**Get the average signal level (0..32767) for the audio media channel.
@@ -1893,6 +1925,7 @@ class OpalConnection : public PSafeObject
     P_REMOVE_VIRTUAL(bool, IsConnectionOnHold(bool), false);
     P_REMOVE_VIRTUAL_VOID(ApplyStringOptions(OpalConnection::StringOptions &));
     P_REMOVE_VIRTUAL(bool, OnTransferNotify(const PStringToString &), false);
+    P_REMOVE_VIRTUAL_VOID(OnSwitchedFaxMediaStreams(bool));
 };
 
 #endif // OPAL_OPAL_CONNECTION_H
