@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28162 $
+ * $Revision: 28428 $
  * $Author: rjongbloed $
- * $Date: 2012-08-07 22:59:15 -0500 (Tue, 07 Aug 2012) $
+ * $Date: 2012-10-01 03:38:50 -0500 (Mon, 01 Oct 2012) $
  */
 
 #ifndef OPAL_SIP_SIPEP_H
@@ -863,10 +863,11 @@ class SIPEndPoint : public OpalRTPEndPoint
 
     /**Return the SIPAuthentication for a specific realm.
      */
-    bool GetAuthentication(
-      SIPAuthentication & newAuthentication,
-      SIPAuthentication * oldAuthentication,
-      const SIPURL & proxy,
+    SIP_PDU::StatusCodes HandleAuthentication(
+      SIPAuthentication * & authentication,
+      unsigned & authenticateErrors,
+      const SIP_PDU & response,
+      const SIPURL & proxyOverride,
       const PString & username,
       const PString & password
     );
