@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27589 $
+ * $Revision: 29080 $
  * $Author: rjongbloed $
- * $Date: 2012-05-07 23:01:54 -0500 (Mon, 07 May 2012) $
+ * $Date: 2013-02-12 18:52:05 -0600 (Tue, 12 Feb 2013) $
  */
 
 #ifndef OPAL_SIP_SIPPDU_H
@@ -234,6 +234,7 @@ class SIPURLList : public std::list<SIPURL>
       bool reversed = false
     );
     PString ToString() const;
+    friend ostream & operator<<(ostream & strm, const SIPURLList & urls);
 };
 
 
@@ -646,8 +647,7 @@ class SIP_PDU : public PSafeObject
 
     PString CreateVia(
       SIPEndPoint & endpoint,
-      const OpalTransport & transport,
-      SIPConnection * connection = NULL
+      const OpalTransport & transport
     );
 
     /**Read PDU from the specified transport.
@@ -820,6 +820,7 @@ struct SIPParameters
   PCaselessString m_proxyAddress;
   PCaselessString m_addressOfRecord;
   PCaselessString m_contactAddress;
+  PCaselessString m_interface;
   SIPMIMEInfo     m_mime;
   PString         m_authID;
   PString         m_password;
@@ -936,8 +937,9 @@ class SIPTransaction : public SIP_PDU
 };
 
 
-#define OPAL_PROXY_PARAM    "OPAL-proxy"
-#define OPAL_LOCAL_ID_PARAM "OPAL-local-id"
+#define OPAL_PROXY_PARAM     "OPAL-proxy"
+#define OPAL_LOCAL_ID_PARAM  "OPAL-local-id"
+#define OPAL_INTERFACE_PARAM "OPAL-interface"
 
 
 /////////////////////////////////////////////////////////////////////////
