@@ -38,9 +38,9 @@
  * Initial implementation of MPEG4 codec plugin using ffmpeg.
  * Untested under Windows or H.323
  *
- * $Revision: 25138 $
+ * $Revision: 28116 $
  * $Author: rjongbloed $
- * $Date: 2011-02-20 20:31:59 -0600 (Sun, 20 Feb 2011) $
+ * $Date: 2012-07-30 22:59:39 -0500 (Mon, 30 Jul 2012) $
  */
 
 /*
@@ -60,6 +60,8 @@
 #include "plugin-config.h"
 #endif
 
+#include "dyna.h"
+#include "rtpframe.h"
 
 #include <codec/opalplugin.hpp>
 
@@ -68,10 +70,6 @@ PLUGIN_CODEC_IMPLEMENT(FFMPEG_MPEG4)
 };
 
 
-#include <stdlib.h>
-#include "dyna.h"
-#include "rtpframe.h"
-
 typedef unsigned char BYTE;
 
 // Needed C++ headers
@@ -104,7 +102,7 @@ extern "C" {
 #include <libavcodec/mpegvideo.h>
 
 #else /* LIBAVCODEC_HAVE_SOURCE_DIR */
-#include LIBAVCODEC_HEADER
+#include "../common/ffmpeg.h"
 #endif /* LIBAVCODEC_HAVE_SOURCE_DIR */
 }
 
@@ -130,6 +128,8 @@ extern "C" {
 
 #define MAX_MPEG4_PACKET_SIZE     2048
 
+static const char ProfileAndLevel[] = "Profile & Level";
+
 const static struct mpeg4_profile_level {
     unsigned profileLevel;
     const char* profileName;
@@ -808,7 +808,7 @@ int MPEG4EncoderContext::EncodeFrames(const BYTE * src, unsigned & srcLen,
         }
         else // No IFrame requested, let avcodec decide what to do
         {
-            m_avpicture->pict_type = 0;
+            m_avpicture->pict_type = AV_PICTURE_TYPE_NONE;
         }
 
         // Encode a frame
@@ -1010,7 +1010,7 @@ static int to_normalised_options(const struct PluginCodec_Definition *, void *,
   unsigned targetBitrate = 64000;
 
   for (const char * const * option = *(const char * const * *)parm; *option != NULL; option += 2) {
-      if (STRCMPI(option[0], "CAP RFC3016 Profile Level") == 0)
+      if (STRCMPI(option[0], ProfileAndLevel) == 0)
         profileLevel = atoi(option[1]);
       if (STRCMPI(option[0], PLUGINCODEC_OPTION_FRAME_WIDTH) == 0)
         width = atoi(option[1]);
@@ -1087,7 +1087,7 @@ static int encoder_set_options(
     unsigned targetBitrate = 64000;
     unsigned profileLevel = 1;
     for (int i = 0; options[i] != NULL; i += 2) {
-      if (STRCMPI(options[i], "CAP RFC3016 Profile Level") == 0)
+      if (STRCMPI(options[i], ProfileAndLevel) == 0)
          profileLevel = atoi(options[i+1]);
       else if (STRCMPI(options[i], PLUGINCODEC_OPTION_FRAME_WIDTH) == 0)
         context->SetFrameWidth(atoi(options[i+1]));
@@ -1786,7 +1786,7 @@ static void FreeString(char * str)
 static struct PluginCodec_Option const H245ProfileLevelMPEG4 =
 {
   PluginCodec_IntegerOption,          // Option type
-  "H.245 Profile & Level",            // User visible name
+  ProfileAndLevel,                    // User visible name
   false,                              // User Read/Only flag
   PluginCodec_CustomMerge,            // Merge mode
   "5",                                // Initial value (Simple Profile/Level 5)
