@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27929 $
+ * $Revision: 28422 $
  * $Author: rjongbloed $
- * $Date: 2012-06-28 20:19:31 -0500 (Thu, 28 Jun 2012) $
+ * $Date: 2012-09-28 00:00:45 -0500 (Fri, 28 Sep 2012) $
  */
 
 #ifndef _OpenPhone_MAIN_H
@@ -85,22 +85,13 @@ class MyPCSSEndPoint : public OpalPCSSEndPoint
   public:
     MyPCSSEndPoint(MyManager & manager);
 
-    virtual OpalMediaFormatList GetMediaFormats() const;
     virtual PSoundChannel * CreateSoundChannel(const OpalPCSSConnection & connection, const OpalMediaFormat & mediaFormat, PBoolean isSource);
 
-    MyManager & m_manager;
-
-/*
-    bool TransmitExternalIM(
-      OpalConnection & conn, 
-      const OpalMediaFormat & format, 
-      RTP_IMFrame & frame
-    );
-*/
-
   private:
     virtual PBoolean OnShowIncoming(const OpalPCSSConnection & connection);
     virtual PBoolean OnShowOutgoing(const OpalPCSSConnection & connection);
+
+    MyManager & m_manager;
 };
 
 class IMDialog;
