@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26128 $
+ * $Revision: 28048 $
  * $Author: rjongbloed $
- * $Date: 2011-07-07 21:36:57 -0500 (Thu, 07 Jul 2011) $
+ * $Date: 2012-07-17 22:41:57 -0500 (Tue, 17 Jul 2012) $
  */
 
 #include <ptlib.h>
@@ -50,6 +50,8 @@
 #include <codec/opalplugin.h>
 #include <codec/rfc2833.h>
 
+#include <algorithm>
+
 
 #define DEFINE_G711_CAPABILITY(cls, code, capName) \
 class cls : public H323_G711Capability { \
@@ -162,15 +164,25 @@ unsigned H323Capability::GetRxFramesInPacket() const
 }
 
 
-PBoolean H323Capability::IsMatch(const PASN_Choice & subTypePDU) const
+PBoolean H323Capability::IsMatch(const PASN_Choice & subTypePDU, const PString & mediaPacketization) const
 {
-  return subTypePDU.GetTag() == GetSubType();
+  if (subTypePDU.GetTag() != GetSubType())
+    return false;
+
+  if (mediaPacketization.IsEmpty())
+    return true;
+
+  PStringSet mediaPacketizations = GetMediaFormat().GetMediaPacketizations();
+  if (mediaPacketizations.IsEmpty())
+    return true;
+
+  return mediaPacketizations.Contains(mediaPacketization);
 }
 
 
 PBoolean H323Capability::OnSendingPDU(H245_DataType & /*pdu*/) const
 {
-  GetWritableMediaFormat().SetOptionString(OpalMediaFormat::ProtocolOption(), "H.323");
+  GetWritableMediaFormat().SetOptionString(OpalMediaFormat::ProtocolOption(), PLUGINCODEC_OPTION_PROTOCOL_H323);
   return m_mediaFormat.ToCustomisedOptions();
 }
 
@@ -207,14 +219,14 @@ PBoolean H323Capability::OnReceivedPDU(const H245_Capability & cap)
       capabilityDirection = e_NoDirection;
   }
 
-  GetWritableMediaFormat().SetOptionString(OpalMediaFormat::ProtocolOption(), "H.323");
+  GetWritableMediaFormat().SetOptionString(OpalMediaFormat::ProtocolOption(), PLUGINCODEC_OPTION_PROTOCOL_H323);
   return m_mediaFormat.ToNormalisedOptions();
 }
 
 
 PBoolean H323Capability::OnReceivedPDU(const H245_DataType & /*pdu*/, PBoolean /*receiver*/)
 {
-  GetWritableMediaFormat().SetOptionString(OpalMediaFormat::ProtocolOption(), "H.323");
+  GetWritableMediaFormat().SetOptionString(OpalMediaFormat::ProtocolOption(), PLUGINCODEC_OPTION_PROTOCOL_H323);
   return m_mediaFormat.ToNormalisedOptions();
 }
 
@@ -504,6 +516,14 @@ H323GenericCapabilityInfo::H323GenericCapabilityInfo(const PString & standardId,
 }
 
 
+struct OpalMediaOptionSortByPosition
+{
+  bool operator()(OpalMediaOption const * const & o1, OpalMediaOption const * const & o2)
+  {
+    return o1->GetH245Generic().position < o2->GetH245Generic().position;
+  }
+};
+
 PBoolean H323GenericCapabilityInfo::OnSendingGenericPDU(H245_GenericCapability & pdu,
                                                     const OpalMediaFormat & mediaFormat,
                                                     H323Capability::CommandType type) const
@@ -516,11 +536,13 @@ PBoolean H323GenericCapabilityInfo::OnSendingGenericPDU(H245_GenericCapability &
     pdu.m_maxBitRate = bitRate;
   }
 
+  std::vector<OpalMediaOption const *> reorderedOptions;
   for (PINDEX i = 0; i < mediaFormat.GetOptionCount(); i++) {
     const OpalMediaOption & option = mediaFormat.GetOption(i);
-    OpalMediaOption::H245GenericInfo genericInfo = option.GetH245Generic();
+    const OpalMediaOption::H245GenericInfo & genericInfo = option.GetH245Generic();
     if (genericInfo.mode == OpalMediaOption::H245GenericInfo::None)
       continue;
+
     switch (type) {
       case H323Capability::e_TCS :
         if (genericInfo.excludeTCS)
@@ -536,6 +558,16 @@ PBoolean H323GenericCapabilityInfo::OnSendingGenericPDU(H245_GenericCapability &
         break;
     }
 
+    if (option.AsString() != genericInfo.defaultValue)
+      reorderedOptions.push_back(&option);
+  }
+
+  std::sort(reorderedOptions.begin(), reorderedOptions.end(), OpalMediaOptionSortByPosition()); 
+
+  for (std::vector<OpalMediaOption const *>::iterator it = reorderedOptions.begin(); it != reorderedOptions.end(); ++it) {
+    const OpalMediaOption & option = **it;
+    const OpalMediaOption::H245GenericInfo & genericInfo = option.GetH245Generic();
+
     H245_ArrayOf_GenericParameter & params =
             genericInfo.mode == OpalMediaOption::H245GenericInfo::Collapsing ? pdu.m_collapsing : pdu.m_nonCollapsing;
 
@@ -917,9 +949,9 @@ PBoolean H323GenericAudioCapability::OnReceivedPDU(const H245_AudioCapability &
   return PTrue;
 }
 
-PBoolean H323GenericAudioCapability::IsMatch(const PASN_Choice & subTypePDU) const
+PBoolean H323GenericAudioCapability::IsMatch(const PASN_Choice & subTypePDU, const PString & mediaPacketization) const
 {
-  return H323Capability::IsMatch(subTypePDU) &&
+  return H323Capability::IsMatch(subTypePDU, mediaPacketization) &&
          H323GenericCapabilityInfo::IsMatch((const H245_GenericCapability &)subTypePDU.GetObject());
 }
 
@@ -1002,9 +1034,9 @@ PBoolean H323NonStandardAudioCapability::OnReceivedPDU(const H245_AudioCapabilit
 }
 
 
-PBoolean H323NonStandardAudioCapability::IsMatch(const PASN_Choice & subTypePDU) const
+PBoolean H323NonStandardAudioCapability::IsMatch(const PASN_Choice & subTypePDU, const PString & mediaPacketization) const
 {
-  return H323Capability::IsMatch(subTypePDU) &&
+  return H323Capability::IsMatch(subTypePDU, mediaPacketization) &&
          H323NonStandardCapabilityInfo::IsMatch((const H245_NonStandardParameter &)subTypePDU.GetObject());
 }
 
@@ -1177,9 +1209,9 @@ PBoolean H323NonStandardVideoCapability::OnReceivedPDU(const H245_VideoCapabilit
 }
 
 
-PBoolean H323NonStandardVideoCapability::IsMatch(const PASN_Choice & subTypePDU) const
+PBoolean H323NonStandardVideoCapability::IsMatch(const PASN_Choice & subTypePDU, const PString & mediaPacketization) const
 {
-  return H323Capability::IsMatch(subTypePDU) &&
+  return H323Capability::IsMatch(subTypePDU, mediaPacketization) &&
          H323NonStandardCapabilityInfo::IsMatch((const H245_NonStandardParameter &)subTypePDU.GetObject());
 }
 
@@ -1225,9 +1257,9 @@ PBoolean H323GenericVideoCapability::OnReceivedPDU(const H245_VideoCapability &
   return OnReceivedGenericPDU(GetWritableMediaFormat(), pdu, type);
 }
 
-PBoolean H323GenericVideoCapability::IsMatch(const PASN_Choice & subTypePDU) const
+PBoolean H323GenericVideoCapability::IsMatch(const PASN_Choice & subTypePDU, const PString & mediaPacketization) const
 {
-  return H323Capability::IsMatch(subTypePDU) &&
+  return H323Capability::IsMatch(subTypePDU, mediaPacketization) &&
          H323GenericCapabilityInfo::IsMatch((const H245_GenericCapability &)subTypePDU.GetObject());
 }
 
@@ -1340,9 +1372,10 @@ PBoolean H323ExtendedVideoCapability::OnReceivedPDU(const H245_VideoCapability &
     const H245_VideoCapability & vidCap = extcap.m_videoCapability[i];
     for (PINDEX c = 0; c < allVideoCapabilities.GetSize(); c++) {
       H323VideoCapability & capability = (H323VideoCapability &)allVideoCapabilities[c];
-      if (capability.IsMatch(vidCap) && capability.OnReceivedPDU(vidCap, type)) {
+      if (capability.IsMatch(vidCap, PString::Empty()) && capability.OnReceivedPDU(vidCap, type)) {
         OpalMediaFormat mediaFormat = capability.GetMediaFormat();
         mediaFormat.SetOptionInteger(OpalVideoFormat::ContentRoleMaskOption(), roleMask);
+        mediaFormat.SetOptionString(OpalMediaFormat::ProtocolOption(), PLUGINCODEC_OPTION_PROTOCOL_H323);
         if (mediaFormat.ToNormalisedOptions())
           m_videoFormats += mediaFormat;
       }
@@ -1354,9 +1387,9 @@ PBoolean H323ExtendedVideoCapability::OnReceivedPDU(const H245_VideoCapability &
 }
 
 
-PBoolean H323ExtendedVideoCapability::IsMatch(const PASN_Choice & subTypePDU) const
+PBoolean H323ExtendedVideoCapability::IsMatch(const PASN_Choice & subTypePDU, const PString & mediaPacketization) const
 {
-  if (!H323Capability::IsMatch(subTypePDU))
+  if (!H323Capability::IsMatch(subTypePDU, mediaPacketization))
     return false;
 
   const H245_ExtendedVideoCapability & extcap = (const H245_ExtendedVideoCapability &)subTypePDU.GetObject();
@@ -1414,7 +1447,7 @@ PBoolean H323GenericControlCapability::OnReceivedPDU(const H245_Capability & pdu
 }
 
 
-PBoolean H323GenericControlCapability::IsMatch(const PASN_Choice & subTypePDU) const
+PBoolean H323GenericControlCapability::IsMatch(const PASN_Choice & subTypePDU, const PString &) const
 {
   return H323GenericCapabilityInfo::IsMatch((const H245_GenericCapability &)subTypePDU.GetObject());
 }
@@ -1716,9 +1749,9 @@ PBoolean H323NonStandardDataCapability::OnReceivedPDU(const H245_DataApplication
 }
 
 
-PBoolean H323NonStandardDataCapability::IsMatch(const PASN_Choice & subTypePDU) const
+PBoolean H323NonStandardDataCapability::IsMatch(const PASN_Choice & subTypePDU, const PString & mediaPacketization) const
 {
-  return H323Capability::IsMatch(subTypePDU) &&
+  return H323Capability::IsMatch(subTypePDU, mediaPacketization) &&
          H323NonStandardCapabilityInfo::IsMatch((const H245_NonStandardParameter &)subTypePDU.GetObject());
 }
 
@@ -2012,8 +2045,8 @@ H323Capabilities::H323Capabilities(const H323Connection & connection,
    *  version of the codec against possibly multiple codec with the same 'subtype' such as
    *  e_h263VideoCapability.
    */
-  mediaPacketizations += "RFC2190";  // Always supported
-  mediaPacketizations += OpalPluginCodec_Identifer_H264_Aligned; // Always supported
+  m_mediaPacketizations += "RFC2190";  // Always supported
+  m_mediaPacketizations += OpalPluginCodec_Identifer_H264_Aligned; // Always supported
   const H245_MultiplexCapability * muxCap = NULL;
   if (pdu.HasOptionalField(H245_TerminalCapabilitySet::e_multiplexCapability)) {
     muxCap = &pdu.m_multiplexCapability;
@@ -2028,8 +2061,9 @@ H323Capabilities::H323Capabilities(const H323Connection & connection,
         for (PINDEX i = 0; i < mediaPacket.m_rtpPayloadType.GetSize(); i++) {
           PString mediaPacketization = H323GetRTPPacketization(mediaPacket.m_rtpPayloadType[i]);
           if (!mediaPacketization.IsEmpty()) {
-            mediaPacketizations += mediaPacketization;
-            PTRACE(4, "H323\tH323Capabilities(ctor) Appended mediaPacketization=" << mediaPacketization << ", mediaPacketization count=" << mediaPacketizations.GetSize());
+            m_mediaPacketizations += mediaPacketization;
+            PTRACE(4, "H323\tH323Capabilities(ctor) Appended mediaPacketization="
+                   << mediaPacketization << ", mediaPacketization count=" << m_mediaPacketizations.GetSize());
           }
         } // for ... m_rtpPayloadType.GetSize()
       } // e_rtpPayloadType
@@ -2045,7 +2079,7 @@ H323Capabilities::H323Capabilities(const H323Connection & connection,
     allCapabilities.Add(new H323H239VideoCapability(OpalMediaFormat()));
     allCapabilities.Add(new H323H239ControlCapability());
 #endif
-    allCapabilities.SetMediaPacketizations(mediaPacketizations);
+    allCapabilities.m_mediaPacketizations = m_mediaPacketizations;
     PTRACE(4, "H323\tParsing remote capabilities");
 
     for (PINDEX i = 0; i < pdu.m_capabilityTable.GetSize(); i++) {
@@ -2059,32 +2093,11 @@ H323Capabilities::H323Capabilities(const H323Connection & connection,
             copy->SetCapabilityNumber(pdu.m_capabilityTable[i].m_capabilityTableEntryNumber);
             table.Append(copy);
 
-            if (mediaPacketizations.GetSize() != 0) { // also update the mediaPacketizations option
+            if (!m_mediaPacketizations.IsEmpty()) { // also update the mediaPacketizations option
               OpalMediaFormat & mediaFormat = copy->GetWritableMediaFormat();
-              PString packetizationString = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
-                                            mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption()));
-              if (!packetizationString.IsEmpty()) {
-                // remove packetizations not signaled by the remote party
-                PStringArray packetizations = packetizationString.Tokenise(",");
-                for (PINDEX j = packetizations.GetSize(); j > 0; j--) {
-                  if (!mediaPacketizations.Contains(packetizations[j-1]))
-                    packetizations.RemoveAt(j-1);
-                }
-
-                // construct the new packetization string
-                packetizationString.MakeEmpty();
-                if (packetizations.GetSize() == 0) { // should not happen actually
-                  mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationsOption(), PString::Empty());
-                  mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationOption(),  PString::Empty());
-                }
-                else {
-                  packetizationString = packetizations[0];
-                  for (PINDEX j = 1; j < packetizations.GetSize(); j++)
-                    packetizationString += "," + packetizations[j];
-                  mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationsOption(), packetizationString);
-                  mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationOption(),  packetizations[0]);
-                }
-              }
+              PStringSet intersection;
+              if (PStringSet::Intersection(m_mediaPacketizations, mediaFormat.GetMediaPacketizations(), &intersection))
+                mediaFormat.SetMediaPacketizations(intersection);
             }
           }
         }
@@ -2219,14 +2232,7 @@ PINDEX H323Capabilities::AddMediaFormat(PINDEX descriptorNum,
     if (capability != NULL) {
       capability->GetWritableMediaFormat() = mediaFormat;
       reply = SetCapability(descriptorNum, simultaneous, capability);
-      PString packetizationString = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
-                                    mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption()));
-      if (!packetizationString.IsEmpty()) {
-        PStringArray packetizations = packetizationString.Tokenise(",");
-        for (PINDEX i = 0; i < packetizations.GetSize(); i++) {
-          mediaPacketizations += packetizations[i];
-        }
-      }
+      m_mediaPacketizations.Union(mediaFormat.GetMediaPacketizations());
     }
   }
 
@@ -2325,8 +2331,8 @@ void H323Capabilities::Remove(H323Capability * capability)
 
   unsigned capabilityNumber = capability->GetCapabilityNumber();
 
-  for (PINDEX outer = 0; outer < set.GetSize(); outer++) {
-    for (PINDEX middle = 0; middle < set[outer].GetSize(); middle++) {
+  for (PINDEX outer = 0; outer < set.GetSize(); ) {
+    for (PINDEX middle = 0; middle < set[outer].GetSize(); ) {
       for (PINDEX inner = 0; inner < set[outer][middle].GetSize(); inner++) {
         if (set[outer][middle][inner].GetCapabilityNumber() == capabilityNumber) {
           set[outer][middle].RemoveAt(inner);
@@ -2335,9 +2341,13 @@ void H323Capabilities::Remove(H323Capability * capability)
       }
       if (set[outer][middle].GetSize() == 0)
         set[outer].RemoveAt(middle);
+      else
+        middle++;
     }
     if (set[outer].GetSize() == 0)
       set.RemoveAt(outer);
+    else
+      outer++;
   }
 
   table.Remove(capability);
@@ -2438,79 +2448,62 @@ H323Capability * H323Capabilities::FindCapability(const H245_Capability & cap) c
   for (PINDEX i = 0; i < table.GetSize(); i++) {
     H323Capability & capability = table[i];
 
-    bool found = false;
-    switch (cap.GetTag()) {
-      case H245_Capability::e_receiveAudioCapability :
-      case H245_Capability::e_transmitAudioCapability :
-      case H245_Capability::e_receiveAndTransmitAudioCapability :
-        if (capability.GetMainType() == H323Capability::e_Audio) {
-          const H245_AudioCapability & audio = cap;
-          found = capability.IsMatch(audio);
-        }
-        break;
+    for (PINDEX j = 0; j <= m_mediaPacketizations.GetSize(); ++j) {
+      PString mediaPacketization;
+      if (j < m_mediaPacketizations.GetSize())
+        mediaPacketization = m_mediaPacketizations.GetKeyAt(j);
 
-      case H245_Capability::e_receiveVideoCapability :
-      case H245_Capability::e_transmitVideoCapability :
-      case H245_Capability::e_receiveAndTransmitVideoCapability :
-        if (capability.GetMainType() == H323Capability::e_Video) {
-          const H245_VideoCapability & video = cap;
-          found = capability.IsMatch(video);
-        }
-        break;
+      switch (cap.GetTag()) {
+        case H245_Capability::e_receiveAudioCapability :
+        case H245_Capability::e_transmitAudioCapability :
+        case H245_Capability::e_receiveAndTransmitAudioCapability :
+          if (capability.GetMainType() == H323Capability::e_Audio) {
+            const H245_AudioCapability & audio = cap;
+            if (capability.IsMatch(audio, mediaPacketization))
+              return &capability;
+          }
+          break;
 
-      case H245_Capability::e_receiveDataApplicationCapability :
-      case H245_Capability::e_transmitDataApplicationCapability :
-      case H245_Capability::e_receiveAndTransmitDataApplicationCapability :
-        if (capability.GetMainType() == H323Capability::e_Data) {
-          const H245_DataApplicationCapability & data = cap;
-          found = capability.IsMatch(data.m_application);
-        }
-        break;
+        case H245_Capability::e_receiveVideoCapability :
+        case H245_Capability::e_transmitVideoCapability :
+        case H245_Capability::e_receiveAndTransmitVideoCapability :
+          if (capability.GetMainType() == H323Capability::e_Video) {
+            const H245_VideoCapability & video = cap;
+            if (capability.IsMatch(video, mediaPacketization))
+              return &capability;
+          }
+          break;
 
-      case H245_Capability::e_receiveUserInputCapability :
-      case H245_Capability::e_transmitUserInputCapability :
-      case H245_Capability::e_receiveAndTransmitUserInputCapability :
-        if (capability.GetMainType() == H323Capability::e_UserInput) {
-          const H245_UserInputCapability & ui = cap;
-          found = capability.IsMatch(ui);
-        }
-        break;
+        case H245_Capability::e_receiveDataApplicationCapability :
+        case H245_Capability::e_transmitDataApplicationCapability :
+        case H245_Capability::e_receiveAndTransmitDataApplicationCapability :
+          if (capability.GetMainType() == H323Capability::e_Data) {
+            const H245_DataApplicationCapability & data = cap;
+            if (capability.IsMatch(data.m_application, mediaPacketization))
+              return &capability;
+          }
+          break;
 
-      case H245_Capability::e_receiveRTPAudioTelephonyEventCapability :
-        return FindCapability(H323Capability::e_UserInput, SignalToneRFC2833_SubType);
+        case H245_Capability::e_receiveUserInputCapability :
+        case H245_Capability::e_transmitUserInputCapability :
+        case H245_Capability::e_receiveAndTransmitUserInputCapability :
+          if (capability.GetMainType() == H323Capability::e_UserInput) {
+            const H245_UserInputCapability & ui = cap;
+            if (capability.IsMatch(ui, mediaPacketization))
+              return &capability;
+          }
+          break;
 
-      case H245_Capability::e_genericControlCapability :
-        if (capability.GetMainType() == H323Capability::e_GenericControl)
-          found = capability.IsMatch(cap);
-        break;
-    }
+        case H245_Capability::e_receiveRTPAudioTelephonyEventCapability :
+          return FindCapability(H323Capability::e_UserInput, SignalToneRFC2833_SubType);
 
-    if (found) {
-      if (mediaPacketizations.IsEmpty())
-        return &capability;
-
-      /** If mediaPacketization information is available, use this with the FindCapability() logic.
-       *  Certain codecs, such as H.263, need additional information in order to match the specific
-       *  version of the codec against possibly multiple codec with the same 'subtype' such as
-       *  e_h263VideoCapability.
-       */
-      OpalMediaFormat mediaFormat = capability.GetMediaFormat();
-      PString packetizationString = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
-                                    mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption()));
-      if (packetizationString.IsEmpty())
-        return &capability;
-
-      // require at least one of the media format's packetizations to be in the mediaPacketizations list
-      PStringArray packetizations = packetizationString.Tokenise(",");
-      for (PINDEX j = 0; j < packetizations.GetSize(); j++) {
-        for (PINDEX k = 0; k < mediaPacketizations.GetSize(); k++) {
-          if (mediaPacketizations.GetKeyAt(k) == packetizations[j])
-            return &capability;
-        }
+        case H245_Capability::e_genericControlCapability :
+          if (capability.GetMainType() == H323Capability::e_GenericControl) {
+            if (capability.IsMatch(cap, mediaPacketization))
+              return &capability;
+          }
+          break;
       }
-
-      PTRACE(4, "H323\tUnsupported media packetization " << packetizationString << ", not using capability " << cap);
-      return NULL;
     }
   }
 
@@ -2553,22 +2546,25 @@ H323Capability * H323Capabilities::FindCapability(const H245_Capability & cap) c
 }
 
 
-H323Capability * H323Capabilities::FindCapability(const H245_DataType & dataType) const
+H323Capability * H323Capabilities::FindCapability(const H245_DataType & dataType, const PString & mediaPacketization) const
 {
   for (PINDEX i = 0; i < table.GetSize(); i++) {
     H323Capability & capability = table[i];
     bool checkExact;
     switch (dataType.GetTag()) {
       case H245_DataType::e_audioData :
-        checkExact = capability.GetMainType() == H323Capability::e_Audio && capability.IsMatch((const H245_AudioCapability &)dataType);
+        checkExact = capability.GetMainType() == H323Capability::e_Audio &&
+                     capability.IsMatch((const H245_AudioCapability &)dataType, mediaPacketization);
         break;
 
       case H245_DataType::e_videoData :
-        checkExact = capability.GetMainType() == H323Capability::e_Video && capability.IsMatch((const H245_VideoCapability &)dataType);
+        checkExact = capability.GetMainType() == H323Capability::e_Video &&
+                     capability.IsMatch((const H245_VideoCapability &)dataType, mediaPacketization);
         break;
 
       case H245_DataType::e_data :
-        checkExact = capability.GetMainType() == H323Capability::e_Data && capability.IsMatch(((const H245_DataApplicationCapability &)dataType).m_application);
+        checkExact = capability.GetMainType() == H323Capability::e_Data &&
+                     capability.IsMatch(((const H245_DataApplicationCapability &)dataType).m_application, mediaPacketization);
         break;
 
       default :
@@ -2591,6 +2587,20 @@ H323Capability * H323Capabilities::FindCapability(const H245_DataType & dataType
     }
   }
 
+  /* Hate special cases ... but this is ... expedient.
+     Due to an ambiguity in the TCS syntax, you cannot easily distinguish
+     between H.263 and variants such as H,263+. Thus we have to allow for
+     if we advertise H.263+ the other side may ask for baseline H.263. So
+     we basically need to include all variants of H.263 if the TCS
+     capability is there at all.
+   */
+  if (dataType.GetTag() == H245_DataType::e_videoData &&
+      ((const H245_VideoCapability &)dataType).GetTag() == H245_VideoCapability::e_h263VideoCapability) {
+    H323Capability * capability = FindCapability("*H.263*");
+    if (capability != NULL)
+      return capability;
+  }
+
 #if PTRACING
   if (PTrace::CanTrace(4)) {
     PString tagName;
@@ -2618,7 +2628,8 @@ H323Capability * H323Capabilities::FindCapability(const H245_DataType & dataType
 }
 
 
-H323Capability * H323Capabilities::FindCapability(const H245_ModeElement & modeElement) const
+H323Capability * H323Capabilities::FindCapability(const H245_ModeElement & modeElement,
+                                                  const PString & mediaPacketization) const
 {
   PTRACE(4, "H323\tFindCapability: " << modeElement.m_type.GetTagName());
 
@@ -2628,7 +2639,7 @@ H323Capability * H323Capabilities::FindCapability(const H245_ModeElement & modeE
       case H245_ModeElementType::e_audioMode :
         if (capability.GetMainType() == H323Capability::e_Audio) {
           const H245_AudioMode & audio = modeElement.m_type;
-          if (capability.IsMatch(audio))
+          if (capability.IsMatch(audio, mediaPacketization))
             return &capability;
         }
         break;
@@ -2636,7 +2647,7 @@ H323Capability * H323Capabilities::FindCapability(const H245_ModeElement & modeE
       case H245_ModeElementType::e_videoMode :
         if (capability.GetMainType() == H323Capability::e_Video) {
           const H245_VideoMode & video = modeElement.m_type;
-          if (capability.IsMatch(video))
+          if (capability.IsMatch(video, mediaPacketization))
             return &capability;
         }
         break;
@@ -2644,7 +2655,7 @@ H323Capability * H323Capabilities::FindCapability(const H245_ModeElement & modeE
       case H245_ModeElementType::e_dataMode :
         if (capability.GetMainType() == H323Capability::e_Data) {
           const H245_DataMode & data = modeElement.m_type;
-          if (capability.IsMatch(data.m_application))
+          if (capability.IsMatch(data.m_application, mediaPacketization))
             return &capability;
         }
         break;
@@ -2910,6 +2921,16 @@ OpalMediaFormatList H323Capabilities::GetMediaFormats() const
 {
   OpalMediaFormatList formats;
 
+  PINDEX outerSize = set.GetSize();
+  for (PINDEX outer = 0; outer < outerSize; outer++) {
+    PINDEX middleSize = set[outer].GetSize();
+    for (PINDEX middle = 0; middle < middleSize; middle++) {
+      PINDEX innerSize = set[outer][middle].GetSize();
+      for (PINDEX inner = 0; inner < innerSize; inner++)
+        formats += set[outer][middle][inner].GetMediaFormat();
+    }
+  }
+
   for (PINDEX i = 0; i < table.GetSize(); i++)
     formats += table[i].GetMediaFormat();
 
