@@ -26,9 +26,9 @@
  *
  * Contributor(s): Many thanks to Simon Horne.
  *
- * $Revision: 28181 $
+ * $Revision: 28579 $
  * $Author: rjongbloed $
- * $Date: 2012-08-10 06:49:42 -0500 (Fri, 10 Aug 2012) $
+ * $Date: 2012-11-25 23:46:48 -0600 (Sun, 25 Nov 2012) $
  */
 
 #include <ptlib.h>
@@ -721,14 +721,6 @@ void H323EndPoint::ConsultationTransfer(const PString & primaryCallToken,
 }
 
 
-void H323EndPoint::HoldCall(const PString & token, PBoolean localHold)
-{
-  PSafePtr<H323Connection> connection = FindConnectionWithLock(token);
-  if (connection != NULL)
-    connection->HoldCall(localHold);
-}
-
-
 PBoolean H323EndPoint::IntrudeCall(const PString & remoteParty,
                                unsigned capabilityLevel,
                                void * userData)
