@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25981 $
+ * $Revision: 26270 $
  * $Author: rjongbloed $
- * $Date: 2011-06-08 20:02:41 -0500 (Wed, 08 Jun 2011) $
+ * $Date: 2011-08-03 03:29:01 -0500 (Wed, 03 Aug 2011) $
  */
 
 #include <ptlib.h>
@@ -283,15 +283,21 @@ bool OpalIMMediaStream::WritePacket(RTP_DataFrame & frame)
 
 //////////////////////////////////////////////////////////////////////////////////////////
 
-PAtomicInteger OpalIM::m_messageIdCounter;
-
 OpalIM::OpalIM()
   : m_type(Text)
-  , m_messageId(m_messageIdCounter++)
+  , m_messageId(GetNextMessageId())
 {
   PTRACE(3, "OpalIM\tcreate new IM");
 }
 
+
+PAtomicInteger::IntegerType OpalIM::GetNextMessageId()
+{
+  static PAtomicInteger messageIdCounter;
+  return ++messageIdCounter;
+}
+
+
 //////////////////////////////////////////////////////////////////////////////////////////
 
 OpalIMContext::OpalIMContext()
