@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27984 $
+ * $Revision: 28445 $
  * $Author: rjongbloed $
- * $Date: 2012-07-10 03:24:24 -0500 (Tue, 10 Jul 2012) $
+ * $Date: 2012-10-02 20:11:02 -0500 (Tue, 02 Oct 2012) $
  */
 
 #include <ptlib.h>
@@ -696,38 +696,45 @@ void OpalConnection::AutoStartMediaStreams(bool force)
 
 
 #if OPAL_FAX
-bool OpalConnection::SwitchFaxMediaStreams(bool enableFax)
+bool OpalConnection::SwitchFaxMediaStreams(bool toT38)
 {
   if (m_faxMediaStreamsSwitchState != e_NotSwitchingFaxMediaStreams) {
     PTRACE(2, "OpalCon\tNested call to SwitchFaxMediaStreams on " << *this);
     return false;
   }
 
-  PTRACE(3, "OpalCon\tSwitchFaxMediaStreams to " << (enableFax ? "fax" : "audio") << " on " << *this);
-  OpalMediaFormat format = enableFax ? OpalT38 : OpalG711uLaw;
+  PTRACE(3, "OpalCon\tSwitchFaxMediaStreams to " << (toT38 ? "T.38" : "audio") << " on " << *this);
+  OpalMediaFormat format = toT38 ? OpalT38 : OpalG711uLaw;
   if (!ownerCall.OpenSourceMediaStreams(*this, format.GetMediaType(), 1, format))
     return false;
 
-  m_faxMediaStreamsSwitchState = enableFax ? e_SwitchingToFaxMediaStreams : e_SwitchingFromFaxMediaStreams;
+  m_faxMediaStreamsSwitchState = toT38 ? e_SwitchingToFaxMediaStreams : e_SwitchingFromFaxMediaStreams;
   return true;
 }
 
 
-void OpalConnection::OnSwitchedFaxMediaStreams(bool enabledFax)
+void OpalConnection::OnSwitchedFaxMediaStreams(bool toT38, bool success)
 {
   if (m_faxMediaStreamsSwitchState != e_NotSwitchingFaxMediaStreams) {
     PTRACE(3, "OpalCon\tSwitch of media streams to "
-           << (m_faxMediaStreamsSwitchState == e_SwitchingToFaxMediaStreams ? "fax" : "audio") << ' '
-           << (enabledFax != (m_faxMediaStreamsSwitchState == e_SwitchingToFaxMediaStreams) ? "failed" : "succeeded")
+           << (toT38 ? "T.38" : "audio") << ' '
+           << (success ? "succeeded" : "failed")
            << " on " << *this);
 
     m_faxMediaStreamsSwitchState = e_NotSwitchingFaxMediaStreams;
 
     PSafePtr<OpalConnection> other = GetOtherPartyConnection();
     if (other != NULL)
-      other->OnSwitchedFaxMediaStreams(enabledFax);
+      other->OnSwitchedFaxMediaStreams(toT38, success);
   }
 }
+
+
+bool OpalConnection::OnSwitchingFaxMediaStreams(bool toT38)
+{
+  PTRACE(3, "OpalCon\tRemote switch of media streams to " << (toT38 ? "T.38" : "audio") << " on " << *this);
+  return !toT38;
+}
 #endif
 
 
@@ -1174,6 +1181,24 @@ PBoolean OpalConnection::SetAudioVolume(PBoolean /*source*/, unsigned /*percenta
 }
 
 
+PBoolean OpalConnection::GetAudioVolume(PBoolean /*source*/, unsigned & /*percentage*/)
+{
+  return PFalse;
+}
+
+
+bool OpalConnection::SetAudioMute(bool /*source*/, bool /*mute*/)
+{
+  return false;
+}
+
+
+bool OpalConnection::GetAudioMute(bool /*source*/, bool & /*mute*/)
+{
+  return false;
+}
+
+
 unsigned OpalConnection::GetAudioSignalLevel(PBoolean /*source*/)
 {
     return UINT_MAX;
