@@ -23,9 +23,9 @@
  *
  * Contributor(s): ________________________________________.
  *
- * $Revision: 25806 $
+ * $Revision: 28025 $
  * $Author: rjongbloed $
- * $Date: 2011-05-19 00:14:31 -0500 (Thu, 19 May 2011) $
+ * $Date: 2012-07-13 03:32:47 -0500 (Fri, 13 Jul 2012) $
  */
 
 #include <ptlib.h>
@@ -72,7 +72,6 @@ OpalMediaStream::OpalMediaStream(OpalConnection & conn, const OpalMediaFormat &
   , timestamp(0)
   , marker(true)
   , mismatchedPayloadTypes(0)
-  , mediaPatch(NULL)
   , m_payloadType(mediaFormat.GetPayloadType())
   , m_frameTime(mediaFormat.GetFrameTime())
   , m_frameSize(mediaFormat.GetFrameSize())
@@ -92,12 +91,9 @@ OpalMediaStream::~OpalMediaStream()
 
 void OpalMediaStream::PrintOn(ostream & strm) const
 {
-  strm << GetClass() << '-';
-  if (isSource)
-    strm << "Source";
-  else
-    strm << "Sink";
-  strm << '-' << mediaFormat;
+  strm << GetClass() << '[' << this << "]-"
+       << (isSource ? "Source" : "Sink")
+       << '-' << mediaFormat;
 }
 
 
@@ -107,16 +103,18 @@ OpalMediaFormat OpalMediaStream::GetMediaFormat() const
 }
 
 
-PBoolean OpalMediaStream::UpdateMediaFormat(const OpalMediaFormat & newMediaFormat, bool fromPatch)
+bool OpalMediaStream::UpdateMediaFormat(const OpalMediaFormat & newMediaFormat)
 {
-  PSafeLockReadWrite safeLock(*this);
-  if (!safeLock.IsLocked())
-    return false;
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
 
-  // If we are source, then update the sink side, and vice versa
-  if (!fromPatch && mediaPatch != NULL)
-    return mediaPatch->UpdateMediaFormat(newMediaFormat);
+  return mediaPatch != NULL ? mediaPatch->UpdateMediaFormat(newMediaFormat)
+                            : InternalUpdateMediaFormat(newMediaFormat);
+}
 
+
+bool OpalMediaStream::InternalUpdateMediaFormat(const OpalMediaFormat & newMediaFormat)
+{
   if (!mediaFormat.Update(newMediaFormat))
     return false;
 
@@ -130,25 +128,21 @@ PBoolean OpalMediaStream::UpdateMediaFormat(const OpalMediaFormat & newMediaForm
 
 PBoolean OpalMediaStream::ExecuteCommand(const OpalMediaCommand & command)
 {
-  PSafeLockReadOnly safeLock(*this);
-  if (!safeLock.IsLocked())
-    return false;
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
 
   if (mediaPatch == NULL)
     return false;
 
-  return mediaPatch->ExecuteCommand(command, IsSink());
-}
+  PTRACE(4, "Media\tExecute command \"" << command << "\" on " << *this << " for " << connection);
 
+  if (mediaPatch->ExecuteCommand(command, IsSink()))
+    return true;
 
-void OpalMediaStream::SetCommandNotifier(const PNotifier & notifier)
-{
-  if (!LockReadWrite())
-    return;
+  if (IsSink())
+    return false;
 
-  commandNotifier = notifier;
-
-  UnlockReadWrite();
+  return connection.OnMediaCommand(*this, command);
 }
 
 
@@ -164,9 +158,8 @@ PBoolean OpalMediaStream::Start()
   if (!Open())
     return false;
 
-  PSafeLockReadOnly safeLock(*this);
-  if (!safeLock.IsLocked())
-    return false;
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
 
   if (mediaPatch == NULL)
     return false;
@@ -194,33 +187,22 @@ PBoolean OpalMediaStream::Close()
 
   // Allow for race condition where it is closed in another thread during the above wait
   if (!isOpen) {
+    PTRACE(4, "Media\tAlready closed stream " << *this);
     UnlockReadWrite();
     return false;
   }
 
   isOpen = false;
 
+  InternalClose();
+
   UnlockReadWrite();
 
   connection.OnClosedMediaStream(*this);
-
-  if (mediaPatch != NULL && LockReadWrite()) {
-    PTRACE(4, "Media\tDisconnecting " << *this << " from patch thread " << *mediaPatch);
-    OpalMediaPatch * patch = mediaPatch;
-    mediaPatch = NULL;
-
-    if (IsSink())
-      patch->RemoveSink(this);
-	
-    UnlockReadWrite();
-
-    if (IsSource()) {
-      patch->Close();
-      connection.GetEndPoint().GetManager().DestroyMediaPatch(patch);
-    }
-  }
-
+  SetPatch(NULL);
   connection.RemoveMediaStream(*this);
+
+  PTRACE(5, "Media\tClosed stream " << *this);
   return true;
 }
 
@@ -393,14 +375,11 @@ PBoolean OpalMediaStream::WriteData(const BYTE * buffer, PINDEX length, PINDEX &
 
 PBoolean OpalMediaStream::PushPacket(RTP_DataFrame & packet)
 {
-  OpalMediaPatch * patch = NULL;
-  if (LockReadOnly()) {
-    patch = mediaPatch;
-    UnlockReadOnly();
-  }
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
 
   // OpalMediaPatch::PushFrame() might block, do outside of mutex
-  return patch != NULL && patch->PushFrame(packet);
+  return mediaPatch != NULL && mediaPatch->PushFrame(packet);
 }
 
 
@@ -435,14 +414,17 @@ bool OpalMediaStream::EnableJitterBuffer(bool) const
 
 bool OpalMediaStream::SetPaused(bool pause, bool fromPatch)
 {
-  PSafeLockReadWrite safeLock(*this);
-  if (!safeLock.IsLocked())
-    return false;
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
 
   // If we are source, then update the sink side, and vice versa
   if (!fromPatch && mediaPatch != NULL)
     return mediaPatch->SetPaused(pause);
 
+  PSafeLockReadWrite mutex(*this);
+  if (!mutex.IsLocked())
+    return false;
+
   if (m_paused == pause)
     return false;
 
@@ -456,6 +438,8 @@ bool OpalMediaStream::SetPaused(bool pause, bool fromPatch)
 
 PBoolean OpalMediaStream::SetPatch(OpalMediaPatch * patch)
 {
+  PatchPtr mediaPatch = m_mediaPatch.Set(patch);
+
 #if PTRACING
   if (PTrace::CanTrace(4) && (patch != NULL || mediaPatch != NULL)) {
     ostream & trace = PTrace::Begin(4, __FILE__, __LINE__);
@@ -469,23 +453,11 @@ PBoolean OpalMediaStream::SetPatch(OpalMediaPatch * patch)
   }
 #endif
 
-  PSafeLockReadWrite safeLock(*this);
-  if (!safeLock.IsLocked())
-    return false;
-
-  if (mediaPatch == NULL) {
-    mediaPatch = patch;
-    return true;
-  }
-
-  OpalMediaPatch * oldPatch = mediaPatch;
-  mediaPatch = patch;
-
-  if (IsSink())
-    oldPatch->RemoveSink(this);
-  else {
-    oldPatch->Close();
-    connection.GetEndPoint().GetManager().DestroyMediaPatch(oldPatch);
+  if (mediaPatch != NULL) {
+    if (IsSink())
+      mediaPatch->RemoveSink(this);
+    else
+      mediaPatch->Close();
   }
 
   return true;
@@ -494,24 +466,30 @@ PBoolean OpalMediaStream::SetPatch(OpalMediaPatch * patch)
 
 void OpalMediaStream::AddFilter(const PNotifier & filter, const OpalMediaFormat & stage) const
 {
-  PSafeLockReadOnly safeLock(*this);
-  if (safeLock.IsLocked() && mediaPatch != NULL)
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
+
+  if (mediaPatch != NULL)
     mediaPatch->AddFilter(filter, stage);
 }
 
 
 PBoolean OpalMediaStream::RemoveFilter(const PNotifier & filter, const OpalMediaFormat & stage) const
 {
-  PSafeLockReadOnly safeLock(*this);
-  return safeLock.IsLocked() && mediaPatch != NULL && mediaPatch->RemoveFilter(filter, stage);
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
+
+  return mediaPatch != NULL && mediaPatch->RemoveFilter(filter, stage);
 }
 
 
 #if OPAL_STATISTICS
 void OpalMediaStream::GetStatistics(OpalMediaStatistics & statistics, bool fromPatch) const
 {
-  PSafeLockReadOnly safeLock(*this);
-  if (safeLock.IsLocked() && mediaPatch != NULL && !fromPatch)
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
+
+  if (mediaPatch != NULL && !fromPatch)
     mediaPatch->GetStatistics(statistics, IsSink());
 }
 #endif
@@ -519,7 +497,11 @@ void OpalMediaStream::GetStatistics(OpalMediaStatistics & statistics, bool fromP
 
 void OpalMediaStream::OnStartMediaPatch() 
 { 
-  connection.OnStartMediaPatch(*mediaPatch);
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
+
+  if (mediaPatch != NULL)
+    connection.OnStartMediaPatch(*mediaPatch);
 }
 
 
@@ -536,6 +518,10 @@ OpalMediaStreamPacing::OpalMediaStreamPacing(const OpalMediaFormat & mediaFormat
   , m_frameTime(mediaFormat.GetFrameTime())
   , m_frameSize(mediaFormat.GetFrameSize())
   , m_timeUnits(mediaFormat.GetTimeUnits())
+  /* No more than 1 second of "slip", reset timing if thread does not execute
+     the timeout for a whole second. Prevents too much bunching up of data
+     in "bad" conditions, that really should not happen. */
+  , m_delay(1000)
 {
   PAssert(!(m_isAudio && m_frameSize == 0), PInvalidParameter);
 }
@@ -614,6 +600,19 @@ PBoolean OpalNullMediaStream::WriteData(const BYTE * /*buffer*/, PINDEX length,
 }
 
 
+bool OpalNullMediaStream::SetPaused(bool pause, bool fromPatch)
+{
+  if (!OpalMediaStream::SetPaused(pause, fromPatch))
+    return false;
+
+  // If coming out of pause, restart pacing delay
+  if (!pause)
+    m_delay.Restart();
+
+  return true;
+}
+
+
 PBoolean OpalNullMediaStream::RequiresPatchThread() const
 {
   return m_requiresPatchThread;
@@ -647,6 +646,8 @@ OpalRTPMediaStream::OpalRTPMediaStream(OpalRTPConnection & conn,
      whatever the RTP sender throws at us. For sink, we set it to the
      maximum size based on MTU (or other criteria). */
   defaultDataSize = isSource ? conn.GetEndPoint().GetManager().GetMaxRtpPacketSize() : conn.GetMaxRtpPayloadSize();
+
+  PTRACE(5, "Media\tCreated RTP media session, RTP=" << &rtp);
 }
 
 
@@ -668,17 +669,11 @@ PBoolean OpalRTPMediaStream::Open()
 }
 
 
-PBoolean OpalRTPMediaStream::Close()
+void OpalRTPMediaStream::InternalClose()
 {
-  if (!isOpen)
-    return false;
-    
-  PTRACE(3, "Media\tClosing RTP for " << *this);
-
   // Break any I/O blocks and wait for the thread that uses this object to
   // terminate before we allow it to be deleted.
   rtpSession.Close(IsSource());
-  return OpalMediaStream::Close();
 }
 
 
@@ -701,6 +696,9 @@ bool OpalRTPMediaStream::SetPaused(bool pause, bool fromPatch)
 
 PBoolean OpalRTPMediaStream::ReadPacket(RTP_DataFrame & packet)
 {
+  if (!IsOpen())
+    return false;
+
   if (IsSink()) {
     PTRACE(1, "Media\tTried to read from sink media stream");
     return false;
@@ -716,6 +714,9 @@ PBoolean OpalRTPMediaStream::ReadPacket(RTP_DataFrame & packet)
 
 PBoolean OpalRTPMediaStream::WritePacket(RTP_DataFrame & packet)
 {
+  if (!IsOpen())
+    return false;
+
   if (IsSource()) {
     PTRACE(1, "Media\tTried to write to source media stream");
     return false;
@@ -740,7 +741,8 @@ PBoolean OpalRTPMediaStream::SetDataSize(PINDEX PTRACE_PARAM(dataSize), PINDEX /
 
 PBoolean OpalRTPMediaStream::IsSynchronous() const
 {
-  return false;
+  // Sinks never block, and source with jitter buffer never blocks
+  return IsSource() && maxAudioJitterDelay == 0;
 }
 
 
@@ -752,7 +754,7 @@ PBoolean OpalRTPMediaStream::RequiresPatchThread() const
 
 bool OpalRTPMediaStream::EnableJitterBuffer(bool enab) const
 {
-  if (IsSink())
+  if (!IsOpen() || IsSink() || !RequiresPatchThread())
     return false;
 
   unsigned minJitter, maxJitter;
@@ -772,7 +774,7 @@ bool OpalRTPMediaStream::EnableJitterBuffer(bool enab) const
 
 PBoolean OpalRTPMediaStream::SetPatch(OpalMediaPatch * patch)
 {
-  if (!isOpen || IsSink() || mediaPatch == NULL)
+  if (!isOpen || IsSink())
     return OpalMediaStream::SetPatch(patch);
 
   rtpSession.Close(true);
@@ -935,15 +937,10 @@ bool OpalRawMediaStream::SetChannel(PChannel * chan, bool autoDelete)
 }
 
 
-PBoolean OpalRawMediaStream::Close()
+void OpalRawMediaStream::InternalClose()
 {
-  if (!isOpen)
-    return false;
-
   if (m_channel != NULL)
     m_channel->Close();
-
-  return OpalMediaStream::Close();
 }
 
 
@@ -1046,8 +1043,6 @@ OpalAudioMediaStream::OpalAudioMediaStream(OpalConnection & conn,
   : OpalRawMediaStream(conn, mediaFormat, sessionID, isSource, channel, autoDel)
   , m_soundChannelBuffers(buffers)
   , m_soundChannelBufferTime(bufferTime)
-  , m_failSafeRate(mediaFormat.GetTimeUnits())
-  , m_failSafeDelay(50, 10)
 {
 }
 
@@ -1068,8 +1063,6 @@ OpalAudioMediaStream::OpalAudioMediaStream(OpalConnection & conn,
                        true)
   , m_soundChannelBuffers(buffers)
   , m_soundChannelBufferTime(bufferTime)
-  , m_failSafeRate(mediaFormat.GetTimeUnits())
-  , m_failSafeDelay(50, 10)
 {
 }
 
@@ -1111,18 +1104,6 @@ PBoolean OpalAudioMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameTime)
 }
 
 
-PBoolean OpalAudioMediaStream::WriteData(const BYTE * data, PINDEX length, PINDEX & written)
-{
-  if (!OpalRawMediaStream::WriteData(data, length, written))
-    return false;
-
-  /* Note if the audio device is working correctly, then this actually will have
-     no effect, it will always be delaying 0ms */
-  m_failSafeDelay.Delay(length/sizeof(short)/m_failSafeRate);
-  return true;
-}
-
-
 PBoolean OpalAudioMediaStream::IsSynchronous() const
 {
   return true;
@@ -1181,6 +1162,36 @@ PBoolean OpalVideoMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameTime)
 }
 
 
+bool OpalVideoMediaStream::InternalUpdateMediaFormat(const OpalMediaFormat & newMediaFormat)
+{
+  if (!OpalMediaStream::InternalUpdateMediaFormat(newMediaFormat))
+    return false;
+
+  unsigned width = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameWidthOption(), PVideoFrameInfo::QCIFWidth);
+  unsigned height = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameHeightOption(), PVideoFrameInfo::QCIFHeight);
+
+  if (m_inputDevice != NULL) {
+    if (!m_inputDevice->SetFrameSizeConverter(width, height)) {
+      PTRACE(1, "Media\tCould not set frame size in grabber to " << width << 'x' << height << " in " << mediaFormat);
+      return false;
+    }
+    if (!m_inputDevice->SetFrameRate(mediaFormat.GetClockRate()/mediaFormat.GetFrameTime())) {
+      PTRACE(1, "Media\tCould not set frame rate in grabber to " << (mediaFormat.GetClockRate()/mediaFormat.GetFrameTime()));
+      return false;
+    }
+  }
+
+  if (m_outputDevice != NULL) {
+    if (!m_outputDevice->SetFrameSizeConverter(width, height)) {
+      PTRACE(1, "Media\tCould not set frame size in video display to " << width << 'x' << height << " in " << mediaFormat);
+      return false;
+    }
+  }
+
+  return true;
+}
+
+
 PBoolean OpalVideoMediaStream::Open()
 {
   if (isOpen)
@@ -1226,18 +1237,21 @@ PBoolean OpalVideoMediaStream::Open()
 }
 
 
-PBoolean OpalVideoMediaStream::Close()
+void OpalVideoMediaStream::InternalClose()
 {
-  if (!OpalMediaStream::Close())
-    return false;
+  if (m_inputDevice != NULL) {
+    if (m_autoDeleteInput)
+      m_inputDevice->Close();
+    else
+      m_inputDevice->Stop();
+  }
 
-  if (m_inputDevice != NULL)
-    m_inputDevice->Stop();
-
-  if (m_outputDevice != NULL)
-    m_outputDevice->Stop();
-
-  return true;
+  if (m_outputDevice != NULL) {
+    if (m_autoDeleteOutput)
+      m_outputDevice->Close();
+    else
+      m_outputDevice->Stop();
+  }
 }
 
 
@@ -1270,8 +1284,10 @@ PBoolean OpalVideoMediaStream::ReadData(BYTE * data, PINDEX size, PINDEX & lengt
 
   PINDEX bytesReturned = size - sizeof(OpalVideoTranscoder::FrameHeader);
   unsigned flags = 0;
-  if (!m_inputDevice->GetFrameData((BYTE *)OPAL_VIDEO_FRAME_DATA_PTR(frame), &bytesReturned, flags))
+  if (!m_inputDevice->GetFrameData((BYTE *)OPAL_VIDEO_FRAME_DATA_PTR(frame), &bytesReturned, flags)) {
+    PTRACE(2, "Media\tFailed to grab frame from " << m_inputDevice->GetDeviceName());
     return false;
+  }
 
   PTimeInterval currentGrabTime = PTimer::Tick();
   timestamp += (int)((currentGrabTime - m_lastGrabTime).GetMilliSeconds()*OpalMediaFormat::VideoClockRate/1000);
@@ -1416,13 +1432,9 @@ PBoolean OpalUDPMediaStream::IsSynchronous() const
 }
 
 
-PBoolean OpalUDPMediaStream::Close()
+void OpalUDPMediaStream::InternalClose()
 {
-  if (!isOpen)
-    return false;
-    
   udpTransport.Close();
-  return OpalMediaStream::Close();
 }
 
 
