@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26010 $
+ * $Revision: 27985 $
  * $Author: rjongbloed $
- * $Date: 2011-06-14 01:59:38 -0500 (Tue, 14 Jun 2011) $
+ * $Date: 2012-07-10 03:25:29 -0500 (Tue, 10 Jul 2012) $
  */
 
 #include <ptlib.h>
@@ -53,10 +53,6 @@
 
 OpalMediaPatch::OpalMediaPatch(OpalMediaStream & src)
   : source(src)
-#if OPAL_VIDEO
-  , m_videoDecoder(src.GetMediaFormat().IsTransportable() && src.GetMediaFormat().GetMediaType() == OpalMediaType::Video())
-#endif
-  , m_bypassActive(false)
   , m_bypassToPatch(NULL)
   , m_bypassFromPatch(NULL)
   , patchThread(NULL)
@@ -68,13 +64,7 @@ OpalMediaPatch::OpalMediaPatch(OpalMediaStream & src)
 
 OpalMediaPatch::~OpalMediaPatch()
 {
-  PWaitAndSignal m(patchThreadMutex);
-  inUse.StartWrite();
-  if (patchThread != NULL) {
-    PAssert(patchThread->WaitForTermination(10000), "Media patch thread not terminated.");
-    delete patchThread;
-    patchThread = NULL;
-  }
+  StopThread();
   PTRACE(5, "Patch\tDestroyed media patch " << this);
 }
 
@@ -83,7 +73,8 @@ void OpalMediaPatch::PrintOn(ostream & strm) const
 {
   strm << "Patch[" << this << "] " << source;
 
-  inUse.StartRead();
+  if (!LockReadOnly())
+    return;
 
   if (sinks.GetSize() > 0) {
     strm << " -> ";
@@ -99,7 +90,7 @@ void OpalMediaPatch::PrintOn(ostream & strm) const
     }
   }
 
-  inUse.EndRead();
+  UnlockReadOnly();
 }
 
 void OpalMediaPatch::Start()
@@ -116,11 +107,31 @@ void OpalMediaPatch::Start()
 }
 
 
+void OpalMediaPatch::StopThread()
+{
+  patchThreadMutex.Wait();
+  PThread * thread = patchThread;
+  patchThread = NULL;
+  patchThreadMutex.Signal();
+
+  if (thread == NULL)
+    return;
+
+  if (!thread->IsSuspended()) {
+    PTRACE(4, "Patch\tWaiting for media patch thread to stop " << *this);
+    PAssert(thread->WaitForTermination(10000), "Media patch thread not terminated.");
+  }
+
+  delete thread;
+}
+
+
 void OpalMediaPatch::Close()
 {
   PTRACE(3, "Patch\tClosing media patch " << *this);
 
-  inUse.StartWrite();
+  if (!LockReadWrite())
+    return;
 
   if (m_bypassFromPatch != NULL)
     m_bypassFromPatch->SetBypassPatch(NULL);
@@ -133,33 +144,25 @@ void OpalMediaPatch::Close()
 
   while (sinks.GetSize() > 0) {
     OpalMediaStreamPtr stream = sinks.front().stream;
-    inUse.EndWrite();
+    UnlockReadWrite();
     if (!stream->Close()) {
       // The only way we can get here is if the sink is in the proccess of being closed
-      // but is blocked on the inUse mutex waiting to remove the sink from this patch.
+      // but is blocked on the mutex waiting to remove the sink from this patch.
       // Se we unlock it, and wait for it to do it in the other thread.
       PThread::Sleep(10);
     }
-    inUse.StartWrite();
-  }
-
-  PTRACE(4, "Patch\tWaiting for media patch thread to stop " << *this);
-  {
-    PWaitAndSignal m(patchThreadMutex);
-    if (patchThread != NULL && !patchThread->IsSuspended()) {
-      inUse.EndWrite();
-      PAssert(patchThread->WaitForTermination(10000), "Media patch thread not terminated.");
+    if (!LockReadWrite())
       return;
-    }
   }
+  UnlockReadWrite();
 
-  inUse.EndWrite();
+  StopThread();
 }
 
 
 PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
 {
-  PWriteWaitAndSignal mutex(inUse);
+  PSafeLockReadWrite mutex(*this);
 
   if (PAssertNULL(sinkStream) == NULL)
     return false;
@@ -211,6 +214,7 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
            << " using transcoder " << *sink->primaryCodec << ", data size=" << sinkStream->GetDataSize());
   }
   else {
+    PTRACE(4, "Patch\tCreating two stage transcoders for " << sourceFormat << "->" << destinationFormat << " with ID " << id);
     OpalMediaFormat intermediateFormat;
     if (!OpalTranscoder::FindIntermediateFormat(sourceFormat, destinationFormat,
                                                 intermediateFormat)) {
@@ -240,7 +244,7 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
     if (sink->primaryCodec == NULL || sink->secondaryCodec == NULL)
       return false;
 
-    PTRACE(4, "Patch\tCreated two stage codec " << sourceFormat << "/" << intermediateFormat << "/" << destinationFormat << " with ID " << id);
+    PTRACE(3, "Patch\tCreated two stage codec " << sourceFormat << "/" << intermediateFormat << "/" << destinationFormat << " with ID " << id);
 
     sink->primaryCodec->SetMaxOutputSize(sink->secondaryCodec->GetOptimalDataFrameSize(true));
     sink->primaryCodec->SetSessionID(source.GetSessionID());
@@ -258,6 +262,12 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
            << " and " << *sink->secondaryCodec << ", data size=" << sinkStream->GetDataSize());
   }
 
+  if (sink->secondaryCodec != NULL)
+    sink->secondaryCodec->SetCommandNotifier(PCREATE_NOTIFIER(OnMediaCommand));
+
+  if (sink->primaryCodec != NULL)
+    sink->primaryCodec->SetCommandNotifier(PCREATE_NOTIFIER(OnMediaCommand));
+
   source.SetDataSize(sink->primaryCodec->GetOptimalDataFrameSize(true), destinationFormat.GetFrameTime());
   EnableJitterBuffer();
   return true;
@@ -269,16 +279,17 @@ void OpalMediaPatch::RemoveSink(const OpalMediaStreamPtr & stream)
   if (PAssertNULL(stream) == NULL)
     return;
 
-  PTRACE(3, "Patch\tRemoving media stream sink " << *stream);
+  PTRACE(3, "Patch\tRemoving sink " << *stream << " from " << *this);
 
   bool closeSource = false;
 
-  inUse.StartWrite();
+  if (!LockReadWrite())
+    return;
 
   for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
     if (s->stream == stream) {
       sinks.erase(s);
-      PTRACE(5, "Patch\tRemoved media stream sink " << *stream);
+      PTRACE(5, "Patch\tRemoved sink " << *stream << " from " << *this);
       break;
     }
   }
@@ -289,7 +300,7 @@ void OpalMediaPatch::RemoveSink(const OpalMediaStreamPtr & stream)
       m_bypassFromPatch->SetBypassPatch(NULL);
   }
 
-  inUse.EndWrite();
+  UnlockReadWrite();
 
   if (closeSource  && source.GetPatch() == this)
     source.Close();
@@ -298,7 +309,7 @@ void OpalMediaPatch::RemoveSink(const OpalMediaStreamPtr & stream)
 
 OpalMediaStreamPtr OpalMediaPatch::GetSink(PINDEX i) const
 {
-  PReadWaitAndSignal mutex(inUse);
+  PSafeLockReadOnly mutex(*this);
   return i < sinks.GetSize() ? sinks[i].stream : OpalMediaStreamPtr();
 }
 
@@ -319,7 +330,8 @@ OpalMediaFormat OpalMediaPatch::GetSinkFormat(PINDEX i) const
 
 OpalTranscoder * OpalMediaPatch::GetAndLockSinkTranscoder(PINDEX i) const
 {
-  inUse.StartRead();
+  if (!LockReadOnly())
+    return NULL;
 
   if (i >= sinks.GetSize()) {
     UnLockSinkTranscoder();
@@ -341,14 +353,15 @@ OpalTranscoder * OpalMediaPatch::GetAndLockSinkTranscoder(PINDEX i) const
 
 void OpalMediaPatch::UnLockSinkTranscoder() const
 {
-  inUse.EndRead();
+  UnlockReadOnly();
 }
 
 
 #if OPAL_STATISTICS
 void OpalMediaPatch::GetStatistics(OpalMediaStatistics & statistics, bool fromSink) const
 {
-  inUse.StartRead();
+  if (!LockReadOnly())
+    return;
 
   if (fromSink)
     source.GetStatistics(statistics, true);
@@ -356,7 +369,7 @@ void OpalMediaPatch::GetStatistics(OpalMediaStatistics & statistics, bool fromSi
   if (!sinks.IsEmpty())
     sinks.front().GetStatistics(statistics, !fromSink);
 
-  inUse.EndRead();
+  UnlockReadOnly();
 }
 
 
@@ -406,7 +419,7 @@ OpalMediaPatch::Sink::~Sink()
 
 void OpalMediaPatch::AddFilter(const PNotifier & filter, const OpalMediaFormat & stage)
 {
-  PWriteWaitAndSignal mutex(inUse);
+  PSafeLockReadWrite mutex(*this);
 
   if (source.GetMediaFormat().GetMediaType() != stage.GetMediaType())
     return;
@@ -424,7 +437,7 @@ void OpalMediaPatch::AddFilter(const PNotifier & filter, const OpalMediaFormat &
 
 PBoolean OpalMediaPatch::RemoveFilter(const PNotifier & filter, const OpalMediaFormat & stage)
 {
-  PWriteWaitAndSignal mutex(inUse);
+  PSafeLockReadWrite mutex(*this);
 
   for (PList<Filter>::iterator f = filters.begin(); f != filters.end(); ++f) {
     if (f->notifier == filter && f->stage == stage) {
@@ -441,22 +454,23 @@ PBoolean OpalMediaPatch::RemoveFilter(const PNotifier & filter, const OpalMediaF
 void OpalMediaPatch::FilterFrame(RTP_DataFrame & frame,
                                  const OpalMediaFormat & mediaFormat)
 {
-  inUse.StartRead();
+  if (!LockReadOnly())
+    return;
 
   for (PList<Filter>::iterator f = filters.begin(); f != filters.end(); ++f) {
     if (f->stage.IsEmpty() || f->stage == mediaFormat)
       f->notifier(frame, (INT)this);
   }
 
-  inUse.EndRead();
+  UnlockReadOnly();
 }
 
 
 bool OpalMediaPatch::UpdateMediaFormat(const OpalMediaFormat & mediaFormat)
 {
-  PReadWaitAndSignal mutex(inUse);
+  PSafeLockReadWrite mutex(*this);
 
-  bool atLeastOne = source.UpdateMediaFormat(mediaFormat, true);
+  bool atLeastOne = source.InternalUpdateMediaFormat(mediaFormat);
 
   for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s)
     atLeastOne = s->UpdateMediaFormat(mediaFormat) || atLeastOne;
@@ -468,16 +482,18 @@ bool OpalMediaPatch::UpdateMediaFormat(const OpalMediaFormat & mediaFormat)
 
 PBoolean OpalMediaPatch::ExecuteCommand(const OpalMediaCommand & command, PBoolean fromSink)
 {
-  PReadWaitAndSignal mutex(inUse);
+  PSafeLockReadOnly mutex(*this);
 
-  if (fromSink)
-    return source.ExecuteCommand(command);
+  if (fromSink) {
+    OpalMediaPatch * patch = m_bypassToPatch != NULL ? m_bypassToPatch : this;
+    PTRACE(5, "Patch\tExecute command \"" << command << "\" "
+           << (m_bypassToPatch != NULL ? "bypassed" : "normally") << ' ' << *this);
+    return patch->source.ExecuteCommand(command);
+  }
 
   bool atLeastOne = false;
 
-  PList<Sink> & sinksToUse = m_bypassActive && m_bypassToPatch != NULL ? m_bypassToPatch->sinks : sinks;
-
-  for (PList<Sink>::iterator s = sinksToUse.begin(); s != sinksToUse.end(); ++s) {
+  for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
     if (s->ExecuteCommand(command))
       atLeastOne = true;
   }
@@ -486,22 +502,15 @@ PBoolean OpalMediaPatch::ExecuteCommand(const OpalMediaCommand & command, PBoole
 }
 
 
-void OpalMediaPatch::SetCommandNotifier(const PNotifier & notifier, PBoolean fromSink)
+void OpalMediaPatch::OnMediaCommand(OpalMediaCommand & command, INT)
 {
-  PReadWaitAndSignal mutex(inUse);
-
-  if (fromSink)
-    source.SetCommandNotifier(notifier);
-  else {
-    for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s)
-      s->SetCommandNotifier(notifier);
-  }
+  source.ExecuteCommand(command);
 }
 
 
 bool OpalMediaPatch::SetPaused(bool pause)
 {
-  PReadWaitAndSignal mutex(inUse);
+  PSafeLockReadOnly mutex(*this);
 
   bool atLeastOne = source.SetPaused(pause, true);
 
@@ -530,7 +539,7 @@ bool OpalMediaPatch::OnStartMediaPatch()
 
 bool OpalMediaPatch::EnableJitterBuffer()
 {
-  PReadWaitAndSignal mutex(inUse);
+  PSafeLockReadOnly mutex(*this);
 
   bool enab = m_bypassToPatch == NULL;
 
@@ -588,11 +597,7 @@ void OpalMediaPatch::Main()
       break;
     }
  
-    inUse.StartRead();
-    bool written = DispatchFrame(sourceFrame);
-    inUse.EndRead();
-
-    if (!written) {
+    if (!DispatchFrame(sourceFrame)) {
       PTRACE(4, "Patch\tThread ended because all sink writes failed");
       break;
     }
@@ -637,7 +642,7 @@ void OpalMediaPatch::Main()
 
 bool OpalMediaPatch::SetBypassPatch(OpalMediaPatch * patch)
 {
-  PWriteWaitAndSignal mutex(inUse);
+  PSafeLockReadWrite mutex(*this);
 
   if (!PAssert(m_bypassFromPatch == NULL, PLogicError))
     return false; // Can't be both!
@@ -665,16 +670,12 @@ bool OpalMediaPatch::SetBypassPatch(OpalMediaPatch * patch)
   m_bypassToPatch = patch;
 
 #if OPAL_VIDEO
-  if (m_videoDecoder)
+  OpalMediaFormat format = source.GetMediaFormat();
+  if (format.IsTransportable() && format.GetMediaType() == OpalMediaType::Video())
     source.ExecuteCommand(OpalVideoUpdatePicture());
   else
-    EnableJitterBuffer();
-
-  m_bypassActive = m_bypassToPatch != NULL && !m_videoDecoder;
-#else
-  EnableJitterBuffer();
-  m_bypassActive = m_bypassToPatch != NULL;
 #endif
+    EnableJitterBuffer();
 
   return true;
 }
@@ -682,19 +683,20 @@ bool OpalMediaPatch::SetBypassPatch(OpalMediaPatch * patch)
 
 PBoolean OpalMediaPatch::PushFrame(RTP_DataFrame & frame)
 {
-  PReadWaitAndSignal mutex(inUse);
   return DispatchFrame(frame);
 }
 
 
 bool OpalMediaPatch::DispatchFrame(RTP_DataFrame & frame)
 {
+  if (!LockReadOnly())
+    return false;
+
   if (m_bypassFromPatch != NULL) {
-    PTRACE(3, "Patch\tMedia patch bypass started by " << m_bypassFromPatch << " on " << *this);
-    inUse.EndRead();
+    PTRACE(3, "Patch\tMedia patch bypass started by " << *m_bypassFromPatch << " on " << *this);
+    UnlockReadOnly();
     m_bypassEnded.Wait();
     PTRACE(4, "Patch\tMedia patch bypass ended on " << *this);
-    inUse.StartRead();
     return true;
   }
 
@@ -702,22 +704,21 @@ bool OpalMediaPatch::DispatchFrame(RTP_DataFrame & frame)
 
   bool written = false;
 
-  // This is not an if/else as WriteFrame can change the state of m_bypassActive
-  if (!m_bypassActive) {
+  if (m_bypassToPatch == NULL) {
     for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
       if (s->WriteFrame(frame))
         written = true;
     }
   }
-
-  if (m_bypassActive) {
-    written = false;
+  else {
+    PSafeLockReadOnly guard(*m_bypassToPatch);
     for (PList<Sink>::iterator s = m_bypassToPatch->sinks.begin(); s != m_bypassToPatch->sinks.end(); ++s) {
       if (s->stream->WritePacket(frame))
         written = true;
     }
   }
 
+  UnlockReadOnly();
   return written;
 }
 
@@ -727,16 +728,16 @@ bool OpalMediaPatch::Sink::UpdateMediaFormat(const OpalMediaFormat & mediaFormat
   bool ok;
 
   if (primaryCodec == NULL)
-    ok = stream->UpdateMediaFormat(mediaFormat, true);
+    ok = stream->InternalUpdateMediaFormat(mediaFormat);
   else if (secondaryCodec != NULL && secondaryCodec->GetOutputFormat() == mediaFormat)
     ok = secondaryCodec->UpdateMediaFormats(OpalMediaFormat(), mediaFormat) &&
-         stream->UpdateMediaFormat(secondaryCodec->GetOutputFormat(), true);
+         stream->InternalUpdateMediaFormat(secondaryCodec->GetOutputFormat());
   else if (primaryCodec->GetOutputFormat() == mediaFormat)
     ok = primaryCodec->UpdateMediaFormats(OpalMediaFormat(), mediaFormat) &&
-         stream->UpdateMediaFormat(primaryCodec->GetOutputFormat(), true);
+         stream->InternalUpdateMediaFormat(primaryCodec->GetOutputFormat());
   else
     ok = primaryCodec->UpdateMediaFormats(mediaFormat, OpalMediaFormat()) &&
-         stream->UpdateMediaFormat(primaryCodec->GetInputFormat(), true);
+         stream->InternalUpdateMediaFormat(primaryCodec->GetInputFormat());
 
 #if OPAL_VIDEO
     SetRateControlParameters(stream->GetMediaFormat());
@@ -761,16 +762,6 @@ bool OpalMediaPatch::Sink::ExecuteCommand(const OpalMediaCommand & command)
 }
 
 
-void OpalMediaPatch::Sink::SetCommandNotifier(const PNotifier & notifier)
-{
-  if (secondaryCodec != NULL)
-    secondaryCodec->SetCommandNotifier(notifier);
-
-  if (primaryCodec != NULL)
-    primaryCodec->SetCommandNotifier(notifier);
-}
-
-
 bool OpalMediaPatch::Sink::CannotTranscodeFrame(OpalTranscoder & codec, RTP_DataFrame & frame)
 {
   RTP_DataFrame::PayloadTypes pt = frame.GetPayloadType();
@@ -818,8 +809,6 @@ void OpalMediaPatch::Sink::SetRateControlParameters(const OpalMediaFormat & medi
   if ((mediaFormat.GetMediaType() == OpalMediaType::Video()) && mediaFormat != OpalYUV420P) {
     rateController = NULL;
     PString rc = mediaFormat.GetOptionString(OpalVideoFormat::RateControllerOption());
-    if (rc.IsEmpty() && mediaFormat.GetOptionBoolean(OpalVideoFormat::RateControlEnableOption()))
-      rc = "Standard";
     if (!rc.IsEmpty()) {
       rateController = PFactory<OpalVideoRateController>::CreateInstance(rc);
       if (rateController != NULL) {   
@@ -894,12 +883,6 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
   }
 
 #if OPAL_VIDEO
-  if (!patch.m_bypassActive && patch.m_videoDecoder && patch.m_bypassToPatch != NULL &&
-                dynamic_cast<OpalVideoTranscoder *>(primaryCodec)->WasLastFrameIFrame()) {
-    PTRACE(3, "Patch\tActivating video patch bypass to " << patch.m_bypassToPatch << " on " << patch);
-    patch.m_bypassActive = true;
-  }
-
   if (secondaryCodec == NULL && rateController != NULL) {
     PTRACE(4, "Patch\tPushing " << intermediateFrames.GetSize() << " packet into RC");
     rateController->Push(intermediateFrames, ((OpalVideoTranscoder *)primaryCodec)->WasLastFrameIFrame());
@@ -971,12 +954,17 @@ OpalMediaPatch::Thread::Thread(OpalMediaPatch & p)
 
 OpalPassiveMediaPatch::OpalPassiveMediaPatch(OpalMediaStream & source)
   : OpalMediaPatch(source)
+  , m_started(false)
 {
 }
 
 
 void OpalPassiveMediaPatch::Start()
 {
+  if (m_started)
+    return;
+
+  m_started = true;
   OnStartMediaPatch();
 }
 
