@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26728 $
+ * $Revision: 28445 $
  * $Author: rjongbloed $
- * $Date: 2011-12-01 22:59:14 -0600 (Thu, 01 Dec 2011) $
+ * $Date: 2012-10-02 20:11:02 -0500 (Tue, 02 Oct 2012) $
  */
 
 #include <ptlib.h>
@@ -422,9 +422,8 @@ OpalMediaFormatList OpalTranscoder::GetPossibleFormats(const OpalMediaFormatList
   // Run through the formats connection can do directly and calculate all of
   // the possible formats, including ones via a transcoder
   for (OpalMediaFormatList::const_iterator f = formats.begin(); f != formats.end(); ++f) {
-    OpalMediaFormat format = *f;
-    possibleFormats += format;
-    OpalMediaFormatList srcFormats = GetSourceFormats(format);
+    possibleFormats += *f;
+    OpalMediaFormatList srcFormats = GetSourceFormats(*f);
     for (OpalMediaFormatList::iterator s = srcFormats.begin(); s != srcFormats.end(); ++s) {
       OpalMediaFormatList dstFormats = GetDestinationFormats(*s);
       if (dstFormats.GetSize() > 0) {
