@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28374 $
+ * $Revision: 29118 $
  * $Author: rjongbloed $
- * $Date: 2012-09-18 05:42:39 -0500 (Tue, 18 Sep 2012) $
+ * $Date: 2013-02-15 20:10:30 -0600 (Fri, 15 Feb 2013) $
  */
 
 #include <ptlib.h>
@@ -1180,10 +1180,8 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnReceiveData(RTP_DataFrame
 
         // As per RFC3550 Appendix 8
         diff *= GetJitterTimeUnits(); // Convert to timestamp units
-        long variance = diff - lastTransitTime;
+        long variance = diff > lastTransitTime ? (diff - lastTransitTime) : (lastTransitTime - diff);
         lastTransitTime = diff;
-        if (variance < 0)
-          variance = -variance;
         jitterLevel += variance - ((jitterLevel+(1<<(JitterRoundingGuardBits-1))) >> JitterRoundingGuardBits);
         if (jitterLevel > maximumJitterLevel)
           maximumJitterLevel = jitterLevel;
@@ -2281,6 +2279,11 @@ RTP_Session::SendReceiveStatus RTP_UDP::ReadDataOrControlPDU(BYTE * framePtr,
       // Shouldn't happen, but it does.
       return RTP_Session::e_IgnorePacket;
 
+    case 0 :
+      PTRACE(4, "RTP_UDP\tSession " << sessionID << ", " << channelName
+             << " received UDP packet with no payload.");
+      return e_IgnorePacket;
+
     default:
       PTRACE(1, "RTP_UDP\tSession " << sessionID << ", " << channelName
              << " read error (" << socket.GetErrorNumber(PChannel::LastReadError) << "): "
