@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28428 $
+ * $Revision: 29080 $
  * $Author: rjongbloed $
- * $Date: 2012-10-01 03:38:50 -0500 (Mon, 01 Oct 2012) $
+ * $Date: 2013-02-12 18:52:05 -0600 (Tue, 12 Feb 2013) $
  *
  * This code implements all or part of the following RFCs
  *
@@ -304,13 +304,12 @@ OpalTransport * SIPHandler::GetTransport()
     m_transport = NULL;
   }
 
-  if (m_proxy.IsEmpty()) {
-    // Look for a "proxy" parameter to override default proxy
-    const PStringToString & params = m_remoteAddress.GetParamVars();
-    if (params.Contains(OPAL_PROXY_PARAM)) {
-      m_proxy.Parse(params(OPAL_PROXY_PARAM));
-      m_remoteAddress.SetParamVar(OPAL_PROXY_PARAM, PString::Empty());
-    }
+  const PStringToString & remoteParams = m_remoteAddress.GetParamVars();
+
+  // Look for a "proxy" parameter to override default proxy
+  if (m_proxy.IsEmpty() && remoteParams.Contains(OPAL_PROXY_PARAM)) {
+    m_proxy.Parse(remoteParams(OPAL_PROXY_PARAM));
+    m_remoteAddress.SetParamVar(OPAL_PROXY_PARAM, PString::Empty());
   }
 
   SIPURL url;
@@ -321,8 +320,11 @@ OpalTransport * SIPHandler::GetTransport()
     url.AdjustToDNS();
   }
 
-  // Must specify a network interface or get infinite recursion
-  m_transport = endpoint.CreateTransport(url, "*");
+  PString localInterface = remoteParams(OPAL_INTERFACE_PARAM);
+  if (localInterface.IsEmpty())
+    localInterface = "*"; // Must specify a network interface or get infinite recursion
+
+  m_transport = endpoint.CreateTransport(url, localInterface);
   return m_transport;
 }
 
@@ -408,15 +410,27 @@ void SIPHandler::OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & resp
 
   m_transactions.Remove(&transaction); // Take this transaction out of list
 
-  // And kill all the rest
-  PSafePtr<SIPTransaction> transToGo;
-  while ((transToGo = m_transactions.GetAt(0)) != NULL) {
-    m_transactions.Remove(transToGo);
-    transToGo->Abort();
-  }
+  switch (response.GetStatusCode()) {
+    default :
+      if (responseClass != 2)
+        break;
 
-  // Finally end connect mode on the transport
-  m_transport->SetInterface(transaction.GetInterface());
+    case SIP_PDU::Failure_UnAuthorised :
+    case SIP_PDU::Failure_ProxyAuthenticationRequired :
+    case SIP_PDU::Failure_IntervalTooBrief :
+    case SIP_PDU::Failure_TemporarilyUnavailable:
+      // End connect mode on the transport
+      PString iface = transaction.GetInterface();
+      PTRACE(4, "SIP\tFinalising handlers interface \"" << iface << '"');
+      m_transport->SetInterface(iface);
+
+      // And kill all the rest
+      PSafePtr<SIPTransaction> transToGo;
+      while ((transToGo = m_transactions.GetAt(0)) != NULL) {
+        m_transactions.Remove(transToGo);
+        transToGo->Abort();
+      }
+  }
 
   switch (response.GetStatusCode()) {
     case SIP_PDU::Failure_UnAuthorised :
@@ -716,6 +730,7 @@ void SIPRegisterHandler::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & re
   for (SIPURLList::iterator reply = replyContacts.begin(); reply != replyContacts.end(); ) {
     if (reply->GetHostAddress() == externalAddress) {
       externalAddress.MakeEmpty(); // Clear this so no further action taken
+      m_externalAddress.MakeEmpty();
       ++reply;
     }
     else if (std::find(requestedContacts.begin(), requestedContacts.end(), *reply) != requestedContacts.end())
@@ -749,27 +764,29 @@ void SIPRegisterHandler::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & re
     return;
   }
 
-  // Remember (possibly new) NAT address
-  m_externalAddress == externalAddress;
-
   if (GetExpire() == 0) {
     // If we had discovered we are behind NAT and had unregistered, re-REGISTER with new addresses
-    PTRACE(2, "SIP\tRe-registering with NAT address " << externalAddress);
+    PTRACE(2, "SIP\tRe-registering NAT address change (" << m_contactAddresses << ") to " << externalAddress);
     for (SIPURLList::iterator contact = m_contactAddresses.begin(); contact != m_contactAddresses.end(); ++contact)
       contact->SetHostAddress(externalAddress);
+    m_contactAddresses.unique();
     SetExpire(m_originalExpireTime);
   }
   else {
     /* If we got here then we have done a successful register, but registrar indicated
        that we are behind firewall. Unregister what we just registered */
-    PTRACE(2, "SIP\tRemote indicated change of REGISTER Contact header required due to NAT");
     for (SIPURLList::iterator contact = replyContacts.begin(); contact != replyContacts.end(); ++contact)
       contact->GetFieldParameters().Remove("expires");
+    PTRACE(2, "SIP\tRemote indicated change of REGISTER Contact address(s) (" << replyContacts
+           << ") required due to NAT address " << externalAddress << ", previous=" << m_externalAddress);
     m_contactAddresses = replyContacts;
     SetExpire(0);
   }
 
-  SendRequest(previousState);
+  // Remember (possibly new) NAT address
+  m_externalAddress == externalAddress;
+
+  SendRequest(Refreshing);
   SendStatus(SIP_PDU::Information_Trying, previousState);
 }
 
@@ -842,8 +859,11 @@ void SIPRegisterHandler::UpdateParameters(const SIPRegister::Params & params)
   if (params.m_expire > 0)
     SetExpire(m_parameters.m_expire = params.m_expire);
 
+  m_parameters.m_compatibility = params.m_compatibility;
   m_parameters.m_contactAddress = params.m_contactAddress;
   m_contactAddresses.clear();
+
+  PTRACE(4, "SIP\tREGISTER parameters updated.");
 }
 
 
@@ -1260,7 +1280,12 @@ class SIPMwiEventPackageHandler : public SIPEventPackageHandler
 
     PMIMEInfo info(request.GetEntityBody());
 
-    PString account = info.Get("Message-Account", handler.GetAddressOfRecord().AsString());
+    const SIPURL & aor = handler.GetAddressOfRecord();
+    PString account = info.Get("Message-Account");
+    SIPURL accountURI(account);
+    if (account.IsEmpty() || aor.GetUserName() == account ||
+            (accountURI.GetUserName() == "asterisk" && accountURI.GetHostName() == aor.GetHostName()))
+      account = aor.AsString();
 
     bool nothingSent = true;
     for (PINDEX z = 0 ; z < PARRAYSIZE(validMessageClasses); z++) {
@@ -2086,15 +2111,16 @@ SIPOptionsHandler::SIPOptionsHandler(SIPEndPoint & endpoint, const SIPOptions::P
 
   m_offlineExpireTime = 0; // No retries for offline, just give up
 
+  /* The "singleton" operation of the OPTION command is executed as though we
+     are unsubscribing from an existing subscription, so when completed the
+     handler is disposed of. So, we need to fake the subscribe state. */
   SetState(Subscribed);
+  m_receivedResponse = true;
 }
 
 
 SIPTransaction * SIPOptionsHandler::CreateTransaction(OpalTransport & transport)
 { 
-  if (GetState() == Unsubscribing)
-    return NULL;
-
   return new SIPOptions(endpoint, transport, GetCallID(), m_parameters);
 }
 
