@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28428 $
+ * $Revision: 29081 $
  * $Author: rjongbloed $
- * $Date: 2012-10-01 03:38:50 -0500 (Mon, 01 Oct 2012) $
+ * $Date: 2013-02-12 18:54:50 -0600 (Tue, 12 Feb 2013) $
  */
 
 #include <ptlib.h>
@@ -223,12 +223,15 @@ void SIPEndPoint::NATBindingRefresh(PTimer &, INT)
            GetManager().GetNatMethod(transport->GetRemoteAddress().GetHostName()) == NULL)
         continue;
 
+      PTRACE(4, "SIP\tNAT Binding refresh for " << handler->GetMethod()
+             << " to " << handler->GetAddressOfRecord() << ", id=" << handler->GetCallID());
       switch (natMethod) {
-
         case Options:
           {
             SIPOptions::Params params;
+            params.m_addressOfRecord = handler->GetAddressOfRecord();
             params.m_remoteAddress = transport->GetRemoteAddress().GetHostName();
+            params.m_interface = transport->GetInterface();
             SendOPTIONS(params);
           }
           break;
@@ -253,7 +256,7 @@ OpalTransport * SIPEndPoint::CreateTransport(const SIPURL & remoteURL, const PSt
   if (remoteAddress.IsEmpty()) {
     if (GetRegistrationsCount() == 0) {
       PTRACE(1, "SIP\tCannot use tel URI with phone-context or existing registration.");
-      return false;
+      return NULL;
     }
     remoteAddress = SIPURL(GetRegistrations()[0]).GetHostAddress();
   }
@@ -886,7 +889,7 @@ PBoolean SIPEndPoint::OnReceivedINVITE(OpalTransport & transport, SIP_PDU * requ
   SIPConnection *connection = CreateConnection(*call,
                                                SIPURL::GenerateTag(),
                                                NULL,
-                                               SIPURL(),
+                                               mime.GetFrom(),
                                                newTransport,
                                                request);
   if (!AddConnection(connection)) {
@@ -1506,7 +1509,7 @@ bool SIPEndPoint::SendOPTIONS(const SIPOptions::Params & newParams)
 
   PSafePtr<SIPHandler> handler = new SIPOptionsHandler(*this, params);
   activeSIPHandlers.Append(handler);
-  return handler->ActivateState(SIPHandler::Subscribing);
+  return handler->ActivateState(SIPHandler::Unsubscribing);
 }
 
 
