@@ -23,9 +23,9 @@
  *
  * Contributor(s): Vyacheslav Frolov.
  *
- * $Revision: 27149 $
+ * $Revision: 28445 $
  * $Author: rjongbloed $
- * $Date: 2012-03-07 18:32:36 -0600 (Wed, 07 Mar 2012) $
+ * $Date: 2012-10-02 20:11:02 -0500 (Tue, 02 Oct 2012) $
  */
 
 #include <ptlib.h>
@@ -114,7 +114,7 @@ class T38PseudoRTP_Handler : public RTP_Encoding
 
       rtpUDP->SetJitterBufferSize(0, 0);
       m_consecutiveBadPackets  = 0;
-      m_oneGoodPacket          = false;
+      m_awaitingGoodPacket     = true;
       m_expectedSequenceNumber = 0;
       m_secondaryPacket        = -1;
 
@@ -399,27 +399,34 @@ class T38PseudoRTP_Handler : public RTP_Encoding
 
       PPER_Stream rawData(thisUDPTL, pduSize);
 
-      // Decode the PDU
-      if (!m_receivedPacket.Decode(rawData)) {
-  #if PTRACING
-        if (m_oneGoodPacket)
-          PTRACE(2, "T38_UDPTL\tRaw data decode failure:\n  "
-                 << setprecision(2) << rawData << "\n  UDPTL = "
-                 << setprecision(2) << m_receivedPacket);
-        else
-          PTRACE(2, "T38_UDPTL\tRaw data decode failure: " << rawData.GetSize() << " bytes.");
+      // Decode the PDU, but not if still receiving RTP
+      bool decodeBad = !m_receivedPacket.Decode(rawData);
+      if (decodeBad || (m_awaitingGoodPacket && m_receivedPacket.m_seq_number >= 32768)) {
+        if (++m_consecutiveBadPackets > 1000) {
+          PTRACE(1, "T38_UDPTL\tRaw data decode failed 1000 times, remote probably not switched from audio, aborting!");
+          return RTP_Session::e_AbortTransport;
+        }
+
+ #if PTRACING
+        static const unsigned Level = 2;
+        if (PTrace::CanTrace(Level)) {
+          ostream & trace = PTrace::Begin(Level, __FILE__, __LINE__);
+          trace << "T38_UDPTL\t";
+          if (m_awaitingGoodPacket)
+            trace << "Probable RTP packet: " << rawData.GetSize() << " bytes.";
+          else
+            trace << "Raw data decode failure:\n  "
+                  << setprecision(2) << rawData << "\n  UDPTL = "
+                  << setprecision(2) << m_receivedPacket;
+          trace << PTrace::End;
+        }
   #endif
 
-        m_consecutiveBadPackets++;
-        if (m_consecutiveBadPackets < 1000)
-          return RTP_Session::e_IgnorePacket;
-
-        PTRACE(1, "T38_UDPTL\tRaw data decode failed 1000 times, remote probably not switched from audio, aborting!");
-        return RTP_Session::e_AbortTransport;
+        return RTP_Session::e_IgnorePacket;
       }
 
-      PTRACE_IF(3, !m_oneGoodPacket, "T38_UDPTL\tFirst decoded UDPTL packet");
-      m_oneGoodPacket = true;
+      PTRACE_IF(3, m_awaitingGoodPacket, "T38_UDPTL\tFirst decoded UDPTL packet");
+      m_awaitingGoodPacket = false;
       m_consecutiveBadPackets = 0;
 
       PTRACE(5, "T38_UDPTL\tDecoded UDPTL packet:\n  " << setprecision(2) << m_receivedPacket);
@@ -448,7 +455,7 @@ class T38PseudoRTP_Handler : public RTP_Encoding
 
   protected:
     int             m_consecutiveBadPackets;
-    bool            m_oneGoodPacket;
+    bool            m_awaitingGoodPacket;
     T38_UDPTLPacket m_receivedPacket;
     unsigned        m_expectedSequenceNumber;
     int             m_secondaryPacket;
@@ -559,6 +566,7 @@ OpalMediaFormatList OpalFaxEndPoint::GetMediaFormats() const
   OpalMediaFormatList formats;
   formats += OpalT38;
   formats += TIFF_File_FormatName;
+PTRACE(4, "OpalFaxEndPoint\tGetMediaFormats for " << *this << "\n    " << setfill(',') << formats << setfill(' '));
   return formats;
 }
 
@@ -590,7 +598,7 @@ OpalFaxConnection::OpalFaxConnection(OpalCall        & call,
 
   m_switchTimer.SetNotifier(PCREATE_NOTIFIER(OnSwitchTimeout));
 
-  PTRACE(3, "FAX\tCreated FAX connection with token \"" << callToken << "\","
+  PTRACE(3, "FAX\tCreated fax connection with token \"" << callToken << "\","
             " receiving=" << receiving << ","
             " disabledT38=" << disableT38 << ","
             " filename=\"" << filename << '"');
@@ -814,24 +822,29 @@ void OpalFaxConnection::OnSwitchTimeout(PTimer &, INT)
 }
 
 
-bool OpalFaxConnection::SwitchFaxMediaStreams(bool enableFax)
+bool OpalFaxConnection::SwitchFaxMediaStreams(bool toT38)
 {
   PSafePtr<OpalConnection> other = GetOtherPartyConnection();
-  if (other != NULL && other->SwitchFaxMediaStreams(enableFax))
+  if (other != NULL && other->SwitchFaxMediaStreams(toT38))
     return true;
 
-  PTRACE(1, "FAX\tMode change request to " << (enableFax ? "fax" : "audio") << " failed");
+  PTRACE(1, "FAX\tMode change request to " << (toT38 ? "T.38" : "audio") << " failed");
   return false;
 }
 
 
-void OpalFaxConnection::OnSwitchedFaxMediaStreams(bool enabledFax)
+void OpalFaxConnection::OnSwitchedFaxMediaStreams(bool toT38, bool success)
 {
-  if (enabledFax) {
-    PTRACE(3, "FAX\tMode change request to fax succeeded");
+  if (!toT38) {
+    PTRACE(3, "FAX\tMode change request to audio");
+    return;
+  }
+
+  if (success) {
+    PTRACE(3, "FAX\tMode change request to T.38 succeeded");
   }
   else {
-    PTRACE(4, "FAX\tMode change request to fax failed, falling back to G.711");
+    PTRACE(4, "FAX\tMode change request to T.38 failed, falling back to G.711");
     if (m_stringOptions.GetBoolean(OPAL_NO_G111_FAX))
       OnFaxCompleted(true);
     else {
@@ -844,6 +857,12 @@ void OpalFaxConnection::OnSwitchedFaxMediaStreams(bool enabledFax)
 }
 
 
+bool OpalFaxConnection::OnSwitchingFaxMediaStreams(bool toT38)
+{
+  return !(toT38 && m_disableT38);
+}
+
+
 void OpalFaxConnection::OpenFaxStreams(PThread &, INT)
 {
   if (LockReadWrite()) {
